#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: application_sets_v1_info
short_description: Information module for Application Sets V1
description:
- Get all Application Sets V1.
- Get appllication-sets by offset/limit or by name.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  offset:
    description:
    - Offset query parameter.
    type: float
  limit:
    description:
    - Limit query parameter.
    type: float
  name:
    description:
    - Name query parameter.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Application Policy GetApplicationSetsV1
  description: Complete reference of the GetApplicationSetsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-application-sets
notes:
  - SDK Method used are
    application_policy.ApplicationPolicy.get_application_sets_v1,

  - Paths used are
    get /dna/intent/api/v1/application-policy-application-set,

"""

EXAMPLES = r""" # yamllint disable-line syntax
- name: Get all Application Sets V1
  cisco.dnac.application_sets_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
    name: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "id": "string",
        "identitySource": {
          "id": "string",
          "type": "string"
        },
        "name": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: endpoint_analytics_cmdb_endpoints_v1
short_description: Resource module for Endpoint Analytics Cmdb Endpoints V1
description:
- Manage operation create of the resource Endpoint Analytics Cmdb Endpoints V1.
- Processes incoming CMDB endpoints data and imports the same in AI Endpoint Analytics.
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Endpoint Analytics Cmdb Endpoints's payload.
    elements: dict
    suboptions:
      assetTag:
        description: Asset tag.
        type: str
      department:
        description: Department that asset belongs to.
        type: str
      displayName:
        description: Display name of the asset.
        type: str
      lastUpdateTimestamp:
        description: Last update timestamp in epoch milliseconds.
        type: int
      location:
        description: Location of the asset.
        type: str
      macAddress:
        description: MAC address of the endpoint.
        type: str
      managedBy:
        description: Asset managed by.
        type: str
      model:
        description: Asset model.
        type: str
      modelCategory:
        description: Category of the model.
        type: str
      serialNumber:
        description: Serial number of the endpoint.
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for AI Endpoint Analytics ProcessCMDBEndpointsV1
  description: Complete reference of the ProcessCMDBEndpointsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!process-cmdb-endpoints
notes:
  - SDK Method used are
    ai_endpoint_analytics.AIEndpointAnalytics.process_cmdb_endpoints_v1,

  - Paths used are
    post /dna/intent/api/v1/endpoint-analytics/cmdb/endpoints,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.endpoint_analytics_cmdb_endpoints_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    payload:
    - assetTag: string
      department: string
      displayName: string
      lastUpdateTimestamp: 0
      location: string
      macAddress: string
      managedBy: string
      model: string
      modelCategory: string
      serialNumber: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: client_health_info
short_description: Information module for Client Health Info
description:
- This module represents an alias of the module client_health_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  timestamp:
    description:
    - Timestamp query parameter. Epoch time(in milliseconds) when the Client health data is required.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Clients GetOverallClientHealthV1
  description: Complete reference of the GetOverallClientHealthV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-overall-client-health
notes:
  - SDK Method used are
    clients.Clients.get_overall_client_health_v1,

  - Paths used are
    get /dna/intent/api/v1/client-health,
  - It should be noted that this module is an alias of client_health_v1_info

"""

EXAMPLES = r"""
- name: Get all Client Health Info
  cisco.dnac.client_health_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    timestamp: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": [
        {
          "siteId": "string",
          "scoreDetail": [
            {
              "scoreCategory": {
                "scoreCategory": "string",
                "value": "string"
              },
              "scoreValue": 0,
              "clientCount": 0,
              "clientUniqueCount": 0,
              "maintenanceAffectedClientCount": 0,
              "randomMacCount": 0,
              "duidCount": 0,
              "starttime": 0,
              "endtime": 0,
              "connectedToUdnCount": 0,
              "unconnectedToUdnCount": 0,
              "scoreList": [
                {
                  "scoreCategory": {
                    "scoreCategory": "string",
                    "value": "string"
                  },
                  "scoreValue": 0,
                  "clientCount": 0,
                  "clientUniqueCount": 0,
                  "maintenanceAffectedClientCount": 0,
                  "randomMacCount": 0,
                  "duidCount": 0,
                  "starttime": 0,
                  "endtime": 0,
                  "connectedToUdnCount": 0,
                  "unconnectedToUdnCount": 0
                }
              ]
            }
          ]
        }
      ]
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2020 Suyeb Ansari (@suyeb786)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_vmbackuppolicy_info
version_added: '1.1.0'
short_description: Fetch Backup Policy Details
description:
    - Get Backup Policy Details.
options:
    name:
        description:
            - Policy Name.
        required: true
        type: str
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    vault_name:
        description:
            - Recovery Service Vault Name.
        required: true
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Suyeb Ansari (@suyeb786)
'''

EXAMPLES = '''
- name: Get backvm policy facts
  azure_rm_backvmuppolicy_info:
    name: 'myBackupPolicy'
    vault_name: 'myVault'
    resource_group: 'myResourceGroup'
'''

RETURN = '''
response:
    description:
        - The response about the current state of the backup policy.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample:  "/subscriptions/xxxxxxx/resourceGroups/resourcegroup_name/ \
            providers/Microsoft.RecoveryServices/vaults/myVault/backupPolicies/myBackup"
        name:
            description:
                - Backup Policy Name.
            returned: always
            type: str
            sample:  "myBackup"
        properties:
            description:
                - The backup policy properties.
            returned: always
            type: dict
            sample: {
                    "backupManagementType": "AzureIaasVM",
                    "schedulePolicy": {
                      "schedulePolicyType": "SimpleSchedulePolicy",
                      "scheduleRunFrequency": "Weekly",
                      "scheduleRunDays": [
                        "Monday",
                        "Wednesday",
                        "Thursday"
                      ],
                      "scheduleRunTimes": [
                        "2018-01-24T10:00:00Z"
                      ],
                      "scheduleWeeklyFrequency": 0
                    },
                    "retentionPolicy": {
                      "retentionPolicyType": "LongTermRetentionPolicy",
                      "weeklySchedule": {
                        "daysOfTheWeek": [
                          "Monday",
                          "Wednesday",
                          "Thursday"
                        ],
                        "retentionTimes": [
                          "2018-01-24T10:00:00Z"
                        ],
                        "retentionDuration": {
                          "count": 1,
                          "durationType": "Weeks"
                        }
                      },
                      "monthlySchedule": {
                        "retentionScheduleFormatType": "Weekly",
                        "retentionScheduleWeekly": {
                          "daysOfTheWeek": [
                            "Wednesday",
                            "Thursday"
                          ],
                          "weeksOfTheMonth": [
                            "First",
                            "Third"
                          ]
                        },
                        "retentionTimes": [
                          "2018-01-24T10:00:00Z"
                        ],
                        "retentionDuration": {
                          "count": 2,
                          "durationType": "Months"
                        }
                      },
                      "yearlySchedule": {
                        "retentionScheduleFormatType": "Weekly",
                        "monthsOfYear": [
                          "February",
                          "November"
                        ],
                        "retentionScheduleWeekly": {
                          "daysOfTheWeek": [
                            "Monday",
                            "Thursday"
                          ],
                          "weeksOfTheMonth": [
                            "Fourth"
                          ]
                        },
                        "retentionTimes": [
                          "2018-01-24T10:00:00Z"
                        ],
                        "retentionDuration": {
                          "count": 4,
                          "durationType": "Years"
                        }
                      }
                    },
                    "timeZone": "Pacific Standard Time",
                    "protectedItemsCount": 0
                   }
        type:
            description:
                - Resource type.
            returned: always
            type: str
            sample:  "Microsoft.RecoveryServices/vaults/backupPolicies"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
import json


class BackupPolicyVMInfo(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            vault_name=dict(
                type='str',
                required=True
            )
        )

        self.resource_group = None
        self.name = None
        self.vault_name = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.url = None
        self.status_code = [200, 202]

        self.query_parameters = {}
        self.query_parameters['api-version'] = '2019-05-13'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        super(BackupPolicyVMInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 supports_tags=True
                                                 )

    def get_url(self):
        return '/subscriptions/' \
               + self.subscription_id \
               + '/resourceGroups/' \
               + self.resource_group \
               + '/providers/Microsoft.RecoveryServices' \
               + '/vaults' + '/' \
               + self.vault_name + '/' \
               + "backupPolicies/" \
               + self.name

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

        self.url = self.get_url()

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        response = self.get_resource()
        changed = False
        self.results['response'] = response
        self.results['changed'] = changed

        return self.results

    def get_resource(self):
        # self.log('Fetch Backup Policy Details {0}'.format(self.))
        try:
            response = self.mgmt_client.query(
                self.url,
                'GET',
                self.query_parameters,
                self.header_parameters,
                None,
                self.status_code,
                600,
                30,
            )
            found = True
        except Exception as e:
            self.log('Backup policy does not exist.')
            self.fail('Error in fetching VM Backup Policy {0}'.format(str(e)))
        if hasattr(response, 'body'):
            response = json.loads(response.body())
        elif hasattr(response, 'context'):
            response = response.context['deserialized_data']
        else:
            self.fail("Create or Updating fail, no match message return, return info as {0}".format(response))

        return response


def main():
    BackupPolicyVMInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_multicast_virtual_networks_v1
short_description: Resource module for Sda Multicast Virtual Networks V1
description:
- Manage operations create, update and delete of the resource Sda Multicast Virtual Networks V1.
- Adds multicast for virtual networks based on user input.
- Deletes a multicast configuration for a virtual network based on id.
- Updates multicast configurations for virtual networks based on user input.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. ID of the multicast configuration.
    type: str
  payload:
    description: Sda Multicast Virtual Networks's payload.
    elements: dict
    suboptions:
      fabricId:
        description: ID of the fabric site this multicast configuration is associated
          with.
        type: str
      ipPoolName:
        description: Name of the IP Pool associated with the fabric site.
        type: str
      ipv4SsmRanges:
        description: IPv4 Source Specific Multicast (SSM) ranges. Allowed ranges are
          from 225.0.0.0/8 to 239.0.0.0/8. SSM ranges should not conflict with ranges
          provided for ASM multicast.
        elements: str
        type: list
      multicastRPs:
        description: Sda Multicast Virtual Networks's multicastRPs.
        elements: dict
        suboptions:
          ipv4Address:
            description: IPv4 address of the RP. For external RP configuration, exactly
              one of ipv4Address or ipv6Address must be provided. For fabric RP, this
              address is allocated by SDA and should not be provided during RP creation
              request and SDA allocated address should be retained in subsequent requests.
            type: str
          ipv4AsmRanges:
            description: IPv4 Any Source Multicast ranges. Comma seperated list of IPv4
              multicast group ranges that will be served by a given Multicast RP. Only
              IPv4 ranges can be provided. For fabric RP, both IPv4 and IPv6 ranges
              can be provided together. For external RP, IPv4 ranges should be provided
              for IPv4 external RP and IPv6 ranges should be provided for IPv6 external
              RP.
            elements: str
            type: list
          ipv6Address:
            description: IPv6 address of the RP. For external RP configuration, exactly
              one of ipv4Address or ipv6Address must be provided. For fabric RP, this
              address is allocated by SDA and should not be provided during RP creation
              request and SDA allocated address should be retained in subsequent requests.
              Ipv6Address can only be provided for virtual networks with dual stack
              (IPv4 + IPv6) multicast pool.
            type: str
          ipv6AsmRanges:
            description: IPv6 Any Source Multicast ranges. Comma seperated list of IPv6
              multicast group ranges that will be served by a given Multicast RP. Only
              IPv6 ranges can be provided. IPv6 ranges can only be provided for dual
              stack multicast pool. For fabric RP, both IPv4 and IPv6 ranges can be
              provided together. For external RP, IPv4 ranges should be provided for
              IPv4 external RP and IPv6 ranges should be provided for IPv6 external
              RP.
            elements: str
            type: list
          isDefaultV4RP:
            description: Specifies whether it is a default IPv4 RP.
            type: bool
          isDefaultV6RP:
            description: Specifies whether it is a default IPv6 RP.
            type: bool
          networkDeviceIds:
            description: IDs of the network devices. This is a required field for fabric
              RPs. There can be maximum of two fabric RPs for a fabric site and these
              are shared across all multicast virtual networks. For configuring two
              fabric RPs in a fabric site both devices must have border roles. Only
              one RP can be configured in scenarios where a fabric edge device is used
              as RP or a dual stack multicast pool is used.
            elements: str
            type: list
          rpDeviceLocation:
            description: Device location of the RP.
            type: str
        type: list
      virtualNetworkName:
        description: Name of the virtual network associated with the fabric site.
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddMulticastVirtualNetworksV1
  description: Complete reference of the AddMulticastVirtualNetworksV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-multicast-virtual-networks
- name: Cisco DNA Center documentation for SDA DeleteMulticastVirtualNetworkByIdV1
  description: Complete reference of the DeleteMulticastVirtualNetworkByIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-multicast-virtual-network-by-id
- name: Cisco DNA Center documentation for SDA UpdateMulticastVirtualNetworksV1
  description: Complete reference of the UpdateMulticastVirtualNetworksV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-multicast-virtual-networks
notes:
  - SDK Method used are
    sda.Sda.add_multicast_virtual_networks_v1,
    sda.Sda.delete_multicast_virtual_network_by_id_v1,
    sda.Sda.update_multicast_virtual_networks_v1,

  - Paths used are
    post /dna/intent/api/v1/sda/multicast/virtualNetworks,
    delete /dna/intent/api/v1/sda/multicast/virtualNetworks/{id},
    put /dna/intent/api/v1/sda/multicast/virtualNetworks,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.sda_multicast_virtual_networks_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - fabricId: string
      ipPoolName: string
      ipv4SsmRanges:
      - string
      multicastRPs:
      - ipv4Address: string
        ipv4AsmRanges:
        - string
        ipv6Address: string
        ipv6AsmRanges:
        - string
        isDefaultV4RP: true
        isDefaultV6RP: true
        networkDeviceIds:
        - string
        rpDeviceLocation: string
      virtualNetworkName: string

- name: Update all
  cisco.dnac.sda_multicast_virtual_networks_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - fabricId: string
      id: string
      ipPoolName: string
      ipv4SsmRanges:
      - string
      multicastRPs:
      - ipv4Address: string
        ipv4AsmRanges:
        - string
        ipv6Address: string
        ipv6AsmRanges:
        - string
        isDefaultV4RP: true
        isDefaultV6RP: true
        networkDeviceIds:
        - string
        rpDeviceLocation: string
      virtualNetworkName: string

- name: Delete by id
  cisco.dnac.sda_multicast_virtual_networks_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: interfaces_query_count_v1
short_description: Resource module for Interfaces Query Count V1
description:
- Manage operation create of the resource Interfaces Query Count V1.
- >
   Gets the total number of interfaces across the Network devices based on the provided complex filters and
   aggregation functions. For detailed information about the usage of the API, please refer to the Open API
   specification document - https //github.com/cisco-en-programmability/catalyst-center-api-
   specs/blob/main/Assurance/CE_Cat_Center_Org-interfaces-1.0.2-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Interfaces Query Count's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Interfaces Query Count's filters.
    elements: dict
    suboptions:
      filters:
        description: Filters.
        elements: str
        type: list
      key:
        description: Key.
        type: str
      logicalOperator:
        description: Logical Operator.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: dict
    type: list
  page:
    description: Interfaces Query Count's page.
    suboptions:
      limit:
        description: Limit.
        type: int
      offset:
        description: Offset.
        type: int
      sortBy:
        description: Interfaces Query Count's sortBy.
        elements: dict
        suboptions:
          name:
            description: Name.
            type: str
          order:
            description: Order.
            type: str
        type: list
    type: dict
  startTime:
    description: Start Time.
    type: int
  views:
    description: Views.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices TheTotalInterfacesCountAcrossTheNetworkDevicesV1
  description: Complete reference of the TheTotalInterfacesCountAcrossTheNetworkDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!the-total-interfaces-count-across-the-network-devices
notes:
  - SDK Method used are
    devices.Devices.the_total_interfaces_count_across_the_network_devices_v1,

  - Paths used are
    post /dna/data/api/v1/interfaces/query/count,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.interfaces_query_count_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
    - function: string
      name: string
    attributes:
    - string
    endTime: 0
    filters:
    - filters:
      - string
      key: string
      logicalOperator: string
      operator: string
      value: {}
    page:
      limit: 0
      offset: 0
      sortBy:
      - name: string
        order: string
    startTime: 0
    views:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_virtualnetworkgateway

version_added: "0.0.1"

short_description: Manage Azure virtual network gateways

description:
    - Create, update or delete a virtual network gateway(VPN Gateway).
    - When creating a VPN Gateway you must provide the name of an existing virtual network.

options:
    resource_group:
        description:
            - Name of a resource group where VPN Gateway exists or will be created.
        required: true
        type: str
    name:
        description:
            - Name of VPN Gateway.
        required: true
        type: str
    state:
        description:
            - State of the VPN Gateway. Use C(present) to create or update VPN gateway and C(absent) to delete VPN gateway.
        default: present
        type: str
        choices:
            - absent
            - present
        required: false
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    virtual_network:
        description:
            - An existing virtual network with which the VPN Gateway will be associated.
            - Required when creating a VPN Gateway.
            - Can be the name of the virtual network.
            - Must be in the same resource group as VPN gateway when specified by name.
            - Can be the resource ID of the virtual network.
            - Can be a dict which contains I(name) and I(resource_group) of the virtual network.
            - Required when creating.
        type: raw
        aliases:
            - virtual_network_name
    ip_configurations:
        description:
            - List of IP configurations.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - Name of the IP configuration.
                type: str
                required: true
            private_ip_allocation_method:
                description:
                    - Private IP allocation method.
                type: str
                choices:
                    - Dynamic
                    - Static
                default: Dynamic
            public_ip_address_name:
                description:
                    - Name of the public IP address. Use 'None' to disable the public IP address.
                type: str
            subnet:
                description:
                    - ID of the gateway subnet for VPN.
                type: str
    gateway_type:
        description:
            - The type of this virtual network gateway.
        default: vpn
        type: str
        choices:
            - vpn
            - express_route
    vpn_type:
        description:
            - The type of this virtual private network.
        default: route_based
        type: str
        choices:
            - route_based
            - policy_based
    enable_bgp:
        description:
            - Whether BGP is enabled for this virtual network gateway or not.
        default: false
        type: bool
    sku:
        description:
            - The reference of the VirtualNetworkGatewaySku resource which represents the SKU selected for Virtual network gateway.
        default: VpnGw1
        type: str
        choices:
            - VpnGw1
            - VpnGw2
            - VpnGw3
            - Standard
            - Basic
            - HighPerformance
    vpn_gateway_generation:
        description:
            - The generation for this VirtualNetworkGateway. Must be C(None) if C(gateway_type) is not VPN.
        default: Generation1
        type: str
        choices:
            - None
            - Generation1
            - Generation2
    bgp_settings:
        description:
            - Virtual network gateway's BGP speaker settings.
        type: dict
        suboptions:
            asn:
                description:
                    - The BGP speaker's ASN.
                type: int
                required: True

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Madhura Naniwadekar (@Madhura-CSI)
'''

EXAMPLES = '''
- name: Create virtual network gateway without bgp settings
  azure_rm_virtualnetworkgateway:
    resource_group: myResourceGroup
    name: myVirtualNetworkGateway
    ip_configurations:
      - name: testipconfig
        private_ip_allocation_method: Dynamic
        public_ip_address_name: testipaddr
    virtual_network: myVirtualNetwork
    tags:
      common: "xyz"

- name: Create virtual network gateway Generation2
  azure_rm_virtualnetworkgateway:
    resource_group: myResourceGroup
    name: myVirtualNetworkGateway
    sku: vpn_gw2
    vpn_gateway_generation: Generation2
    ip_configurations:
      - name: testipconfig
        private_ip_allocation_method: Dynamic
        public_ip_address_name: testipaddr
    virtual_network: myVirtualNetwork
    tags:
      common: "xyz"

- name: Create virtual network gateway with bgp
  azure_rm_virtualnetworkgateway:
    resource_group: myResourceGroup
    name: myVirtualNetworkGateway
    sku: vpn_gw1
    ip_configurations:
      - name: testipconfig
        private_ip_allocation_method: Dynamic
        public_ip_address_name: testipaddr
    enable_bgp: true
    virtual_network: myVirtualNetwork
    bgp_settings:
      asn: 65515
      bgp_peering_address: "169.254.54.209"
    tags:
      common: "xyz"

- name: Delete instance of virtual network gateway
  azure_rm_virtualnetworkgateway:
    resource_group: myResourceGroup
    name: myVirtualNetworkGateway
    state: absent
'''

RETURN = '''
id:
    description:
        - Virtual Network Gateway resource ID.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworkGateways/myV
             irtualNetworkGateway"
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible.module_utils.common.dict_transformations import _snake_to_camel


AZURE_VPN_GATEWAY_OBJECT_CLASS = 'VirtualNetworkGateway'


ip_configuration_spec = dict(
    name=dict(type='str', required=True),
    private_ip_allocation_method=dict(type='str', choices=['Dynamic', 'Static'], default='Dynamic'),
    subnet=dict(type='str'),
    public_ip_address_name=dict(type='str'),
)


sku_spec = dict(
    name=dict(type='str', default='VpnGw1'),
    tier=dict(type='str', default='VpnGw1')
)


bgp_spec = dict(
    asn=dict(type='int', required=True),
)


def vgw_to_dict(vgw):
    results = dict(
        id=vgw.id,
        name=vgw.name,
        location=vgw.location,
        gateway_type=vgw.gateway_type,
        vpn_type=vgw.vpn_type,
        vpn_gateway_generation=vgw.vpn_gateway_generation,
        enable_bgp=vgw.enable_bgp,
        tags=vgw.tags,
        provisioning_state=vgw.provisioning_state,
        sku=dict(
            name=vgw.sku.name,
            tier=vgw.sku.tier
        ),
        bgp_settings=dict(
            asn=vgw.bgp_settings.asn,
            bgp_peering_address=vgw.bgp_settings.bgp_peering_address,
            peer_weight=vgw.bgp_settings.peer_weight
        ) if vgw.bgp_settings else None,
        etag=vgw.etag
    )
    return results


class AzureRMVirtualNetworkGateway(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            location=dict(type='str'),
            ip_configurations=dict(type='list', default=None, elements='dict', options=ip_configuration_spec),
            gateway_type=dict(type='str', default='vpn', choices=['vpn', 'express_route']),
            vpn_type=dict(type='str', default='route_based', choices=['route_based', 'policy_based']),
            vpn_gateway_generation=dict(type='str', default='Generation1', choices=['None', 'Generation1', 'Generation2']),
            enable_bgp=dict(type='bool', default=False),
            sku=dict(type='str', default='VpnGw1', choices=['VpnGw1', 'VpnGw2', 'VpnGw3', 'Standard', 'Basic', 'HighPerformance']),
            bgp_settings=dict(type='dict', options=bgp_spec),
            virtual_network=dict(type='raw', aliases=['virtual_network_name'])
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.location = None
        self.ip_configurations = None
        self.gateway_type = None
        self.vpn_type = None
        self.enable_bgp = None
        self.sku = None
        self.vpn_gateway_generation = None
        self.bgp_settings = None

        required_if = [('state', 'present', ['virtual_network'])]
        self.results = dict(
            changed=False,
            state=dict()
        )

        super(AzureRMVirtualNetworkGateway, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                           required_if=required_if,
                                                           supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        changed = False
        results = dict()
        vgw = None

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            self.location = resource_group.location

        self.virtual_network = self.parse_resource_to_dict(self.virtual_network)
        resource_group = self.get_resource_group(self.resource_group)

        try:
            vgw = self.network_client.virtual_network_gateways.get(self.resource_group, self.name)
            if self.state == 'absent':
                self.log("CHANGED: vnet exists but requested state is 'absent'")
                changed = True
        except ResourceNotFoundError:
            if self.state == 'present':
                self.log("CHANGED: VPN Gateway {0} does not exist but requested state is 'present'".format(self.name))
                changed = True

        if vgw:
            results = vgw_to_dict(vgw)
            if self.state == 'present':
                update_tags, results['tags'] = self.update_tags(results['tags'])
                if update_tags:
                    changed = True
                sku = dict(name=self.sku, tier=self.sku)
                if sku != results['sku']:
                    changed = True
                if self.enable_bgp != results['enable_bgp']:
                    changed = True
                if self.bgp_settings and self.bgp_settings['asn'] != results['bgp_settings']['asn']:
                    changed = True

        self.results['changed'] = changed
        self.results['id'] = results.get('id')

        if self.check_mode:
            return self.results
        if changed:
            if self.state == 'present':
                if not self.sku:
                    self.fail('Parameter error: sku is required when creating a vpn gateway')
                if not self.ip_configurations:
                    self.fail('Parameter error: ip_configurations required when creating a vpn gateway')
                subnet = self.network_models.SubResource(
                    id='/subscriptions/{0}/resourceGroups/{1}/providers/Microsoft.Network/virtualNetworks/{2}/subnets/GatewaySubnet'.format(
                        self.virtual_network['subscription_id'],
                        self.virtual_network['resource_group'],
                        self.virtual_network['name']))

                public_ip_address = self.network_models.SubResource(
                    id='/subscriptions/{0}/resourceGroups/{1}/providers/Microsoft.Network/publicIPAddresses/{2}'.format(
                        self.virtual_network['subscription_id'],
                        self.virtual_network['resource_group'],
                        self.ip_configurations[0]['public_ip_address_name']))

                vgw_ip_configurations = [
                    self.network_models.VirtualNetworkGatewayIPConfiguration(
                        private_ip_allocation_method=ip_config.get('private_ip_allocation_method'),
                        subnet=subnet,
                        public_ip_address=public_ip_address,
                        name='default'
                    ) for ip_config in self.ip_configurations
                ]

                vgw_sku = self.network_models.VirtualNetworkGatewaySku(
                    name=self.sku,
                    tier=self.sku
                )

                vgw_bgp_settings = self.network_models.BgpSettings(
                    asn=self.bgp_settings.get('asn'),
                ) if self.bgp_settings else None
                vgw = self.network_models.VirtualNetworkGateway(
                    location=self.location,
                    ip_configurations=vgw_ip_configurations,
                    gateway_type=_snake_to_camel(self.gateway_type, True),
                    vpn_type=_snake_to_camel(self.vpn_type, True),
                    vpn_gateway_generation=_snake_to_camel(self.vpn_gateway_generation, True),
                    enable_bgp=self.enable_bgp,
                    sku=vgw_sku,
                    bgp_settings=vgw_bgp_settings
                )
                if self.tags:
                    vgw.tags = self.tags
                results = self.create_or_update_vgw(vgw)

            else:
                results = self.delete_vgw()

        if self.state == 'present':
            self.results['id'] = results.get('id')
        return self.results

    def create_or_update_vgw(self, vgw):
        try:
            poller = self.network_client.virtual_network_gateways.begin_create_or_update(self.resource_group, self.name, vgw)
            new_vgw = self.get_poller_result(poller)
            return vgw_to_dict(new_vgw)
        except Exception as exc:
            self.fail("Error creating or updating virtual network gateway {0} - {1}".format(self.name, str(exc)))

    def delete_vgw(self):
        try:
            poller = self.network_client.virtual_network_gateways.begin_delete(self.resource_group, self.name)
            self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error deleting virtual network gateway {0} - {1}".format(self.name, str(exc)))
        return True


def main():
    AzureRMVirtualNetworkGateway()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: pnp_device
short_description: Resource module for Pnp Device
description:
- This module represents an alias of the module pnp_device_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceInfo:
    description: Pnp Device's deviceInfo.
    suboptions:
      description:
        description: Description.
        type: str
      deviceSudiSerialNos:
        description: Device Sudi Serial Nos.
        elements: str
        type: list
      hostname:
        description: Hostname.
        type: str
      macAddress:
        description: Mac Address.
        type: str
      pid:
        description: Pid.
        type: str
      serialNumber:
        description: Serial Number.
        type: str
      siteId:
        description: Site Id.
        type: str
      stack:
        description: Stack.
        type: bool
      stackInfo:
        description: Pnp Device's stackInfo.
        suboptions:
          isFullRing:
            description: Is Full Ring.
            type: bool
          stackMemberList:
            description: Pnp Device's stackMemberList.
            elements: dict
            suboptions:
              hardwareVersion:
                description: Hardware Version.
                type: str
              licenseLevel:
                description: License Level.
                type: str
              licenseType:
                description: License Type.
                type: str
              macAddress:
                description: Mac Address.
                type: str
              pid:
                description: Pid.
                type: str
              priority:
                description: Priority.
                type: float
              role:
                description: Role.
                type: str
              serialNumber:
                description: Serial Number.
                type: str
              softwareVersion:
                description: Software Version.
                type: str
              stackNumber:
                description: Stack Number.
                type: float
              state:
                description: State.
                type: str
              sudiSerialNumber:
                description: Sudi Serial Number.
                type: str
            type: list
          stackRingProtocol:
            description: Stack Ring Protocol.
            type: str
          supportsStackWorkflows:
            description: Supports Stack Workflows.
            type: bool
          totalMemberCount:
            description: Total Member Count.
            type: float
          validLicenseLevels:
            description: Valid License Levels.
            elements: str
            type: list
        type: dict
      sudiRequired:
        description: Is Sudi Required.
        type: bool
      userMicNumbers:
        description: User Mic Numbers.
        elements: str
        type: list
      userSudiSerialNos:
        description: List of Secure Unique Device Identifier (SUDI) serial numbers to
          perform SUDI authorization, Required if sudiRequired is true.
        elements: str
        type: list
      workflowId:
        description: Workflow Id.
        type: str
      workflowName:
        description: Workflow Name.
        type: str
    type: dict
  id:
    description: Id.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Onboarding (PnP) AddDeviceV1
  description: Complete reference of the AddDeviceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-device
- name: Cisco DNA Center documentation for Device Onboarding (PnP) DeleteDeviceByIdFromPnPV1
  description: Complete reference of the DeleteDeviceByIdFromPnPV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-device-by-id-from-pn-p
- name: Cisco DNA Center documentation for Device Onboarding (PnP) UpdateDeviceV1
  description: Complete reference of the UpdateDeviceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-device
notes:
  - SDK Method used are
    device_onboarding_pnp.DeviceOnboardingPnp.add_device,
    device_onboarding_pnp.DeviceOnboardingPnp.delete_device_by_id_from_pnp_v1,
    device_onboarding_pnp.DeviceOnboardingPnp.update_device_v1,

  - Paths used are
    post /dna/intent/api/v1/onboarding/pnp-device,
    delete /dna/intent/api/v1/onboarding/pnp-device/{id},
    put /dna/intent/api/v1/onboarding/pnp-device/{id},
  - It should be noted that this module is an alias of pnp_device_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.pnp_device:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    deviceInfo:
      description: string
      deviceSudiSerialNos:
      - string
      hostname: string
      macAddress: string
      pid: string
      serialNumber: string
      siteId: string
      stack: true
      stackInfo:
        isFullRing: true
        stackMemberList:
        - hardwareVersion: string
          licenseLevel: string
          licenseType: string
          macAddress: string
          pid: string
          priority: 0
          role: string
          serialNumber: string
          softwareVersion: string
          stackNumber: 0
          state: string
          sudiSerialNumber: string
        stackRingProtocol: string
        supportsStackWorkflows: true
        totalMemberCount: 0
        validLicenseLevels:
        - string
      sudiRequired: true
      userMicNumbers:
      - string
      userSudiSerialNos:
      - string
      workflowId: string
      workflowName: string

- name: Update by id
  cisco.dnac.pnp_device:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    deviceInfo:
      hostname: string
      pid: string
      serialNumber: string
      stack: true
      sudiRequired: true
      userSudiSerialNos:
      - string
    id: string

- name: Delete by id
  cisco.dnac.pnp_device:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "_id": "string",
      "deviceInfo": {
        "source": "string",
        "serialNumber": "string",
        "stack": true,
        "mode": "string",
        "state": "string",
        "location": {
          "siteId": "string",
          "address": "string",
          "latitude": "string",
          "longitude": "string",
          "altitude": "string"
        },
        "description": "string",
        "onbState": "string",
        "authenticatedMicNumber": "string",
        "authenticatedSudiSerialNo": "string",
        "capabilitiesSupported": [
          "string"
        ],
        "featuresSupported": [
          "string"
        ],
        "cmState": "string",
        "firstContact": 0,
        "lastContact": 0,
        "macAddress": "string",
        "pid": "string",
        "deviceSudiSerialNos": [
          "string"
        ],
        "lastUpdateOn": 0,
        "workflowId": "string",
        "workflowName": "string",
        "projectId": "string",
        "projectName": "string",
        "deviceType": "string",
        "agentType": "string",
        "imageVersion": "string",
        "fileSystemList": [
          {
            "type": "string",
            "writeable": true,
            "freespace": 0,
            "name": "string",
            "readable": true,
            "size": 0
          }
        ],
        "pnpProfileList": [
          {
            "profileName": "string",
            "discoveryCreated": true,
            "createdBy": "string",
            "primaryEndpoint": {
              "port": 0,
              "protocol": "string",
              "ipv4Address": {},
              "ipv6Address": {},
              "fqdn": "string",
              "certificate": "string"
            },
            "secondaryEndpoint": {
              "port": 0,
              "protocol": "string",
              "ipv4Address": {},
              "ipv6Address": {},
              "fqdn": "string",
              "certificate": "string"
            }
          }
        ],
        "imageFile": "string",
        "httpHeaders": [
          {
            "key": "string",
            "value": "string"
          }
        ],
        "neighborLinks": [
          {
            "localInterfaceName": "string",
            "localShortInterfaceName": "string",
            "localMacAddress": "string",
            "remoteInterfaceName": "string",
            "remoteShortInterfaceName": "string",
            "remoteMacAddress": "string",
            "remoteDeviceName": "string",
            "remotePlatform": "string",
            "remoteVersion": "string"
          }
        ],
        "lastSyncTime": 0,
        "ipInterfaces": [
          {
            "status": "string",
            "macAddress": "string",
            "ipv4Address": {},
            "ipv6AddressList": [
              {}
            ],
            "name": "string"
          }
        ],
        "hostname": "string",
        "authStatus": "string",
        "stackInfo": {
          "supportsStackWorkflows": true,
          "isFullRing": true,
          "stackMemberList": [
            {
              "serialNumber": "string",
              "state": "string",
              "role": "string",
              "macAddress": "string",
              "pid": "string",
              "licenseLevel": "string",
              "licenseType": "string",
              "sudiSerialNumber": "string",
              "hardwareVersion": "string",
              "stackNumber": 0,
              "softwareVersion": "string",
              "priority": 0
            }
          ],
          "stackRingProtocol": "string",
          "validLicenseLevels": [
            "string"
          ],
          "totalMemberCount": 0
        },
        "reloadRequested": true,
        "addedOn": 0,
        "siteId": "string",
        "aaaCredentials": {
          "password": "string",
          "username": "string"
        },
        "userMicNumbers": [
          "string"
        ],
        "userSudiSerialNos": [
          "string"
        ],
        "addnMacAddrs": [
          "string"
        ],
        "preWorkflowCliOuputs": [
          {
            "cli": "string",
            "cliOutput": "string"
          }
        ],
        "tags": {},
        "sudiRequired": true,
        "smartAccountId": "string",
        "virtualAccountId": "string",
        "populateInventory": true,
        "siteName": "string",
        "name": "string"
      },
      "systemResetWorkflow": {
        "_id": "string",
        "state": "string",
        "type": "string",
        "description": "string",
        "lastupdateOn": 0,
        "imageId": "string",
        "currTaskIdx": 0,
        "addedOn": 0,
        "tasks": [
          {
            "state": "string",
            "type": "string",
            "currWorkItemIdx": 0,
            "taskSeqNo": 0,
            "endTime": 0,
            "startTime": 0,
            "workItemList": [
              {
                "state": "string",
                "command": "string",
                "outputStr": "string",
                "endTime": 0,
                "startTime": 0,
                "timeTaken": 0
              }
            ],
            "timeTaken": 0,
            "name": "string"
          }
        ],
        "addToInventory": true,
        "instanceType": "string",
        "endTime": 0,
        "execTime": 0,
        "startTime": 0,
        "useState": "string",
        "configId": "string",
        "name": "string",
        "version": 0,
        "tenantId": "string"
      },
      "systemWorkflow": {
        "_id": "string",
        "state": "string",
        "type": "string",
        "description": "string",
        "lastupdateOn": 0,
        "imageId": "string",
        "currTaskIdx": 0,
        "addedOn": 0,
        "tasks": [
          {
            "state": "string",
            "type": "string",
            "currWorkItemIdx": 0,
            "taskSeqNo": 0,
            "endTime": 0,
            "startTime": 0,
            "workItemList": [
              {
                "state": "string",
                "command": "string",
                "outputStr": "string",
                "endTime": 0,
                "startTime": 0,
                "timeTaken": 0
              }
            ],
            "timeTaken": 0,
            "name": "string"
          }
        ],
        "addToInventory": true,
        "instanceType": "string",
        "endTime": 0,
        "execTime": 0,
        "startTime": 0,
        "useState": "string",
        "configId": "string",
        "name": "string",
        "version": 0,
        "tenantId": "string"
      },
      "workflow": {
        "_id": "string",
        "state": "string",
        "type": "string",
        "description": "string",
        "lastupdateOn": 0,
        "imageId": "string",
        "currTaskIdx": 0,
        "addedOn": 0,
        "tasks": [
          {
            "state": "string",
            "type": "string",
            "currWorkItemIdx": 0,
            "taskSeqNo": 0,
            "endTime": 0,
            "startTime": 0,
            "workItemList": [
              {
                "state": "string",
                "command": "string",
                "outputStr": "string",
                "endTime": 0,
                "startTime": 0,
                "timeTaken": 0
              }
            ],
            "timeTaken": 0,
            "name": "string"
          }
        ],
        "addToInventory": true,
        "instanceType": "string",
        "endTime": 0,
        "execTime": 0,
        "startTime": 0,
        "useState": "string",
        "configId": "string",
        "name": "string",
        "version": 0,
        "tenantId": "string"
      },
      "runSummaryList": [
        {
          "details": "string",
          "historyTaskInfo": {
            "type": "string",
            "workItemList": [
              {
                "state": "string",
                "command": "string",
                "outputStr": "string",
                "endTime": 0,
                "startTime": 0,
                "timeTaken": 0
              }
            ],
            "timeTaken": 0,
            "addnDetails": [
              {
                "key": "string",
                "value": "string"
              }
            ],
            "name": "string"
          },
          "errorFlag": true,
          "timestamp": 0
        }
      ],
      "workflowParameters": {
        "topOfStackSerialNumber": "string",
        "licenseLevel": "string",
        "licenseType": "string",
        "configList": [
          {
            "configParameters": [
              {
                "key": "string",
                "value": "string"
              }
            ],
            "configId": "string"
          }
        ]
      },
      "dayZeroConfig": {
        "config": "string"
      },
      "dayZeroConfigPreview": {},
      "version": 0,
      "tenantId": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2017 Zim Kalinowski, <zikalino@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_containerregistrywebhook_info
version_added: "0.1.2"
short_description: Get Webhook facts.
description:
    - Get facts of Webhook.

options:
    resource_group:
        description:
            - The name of the resource group to which the container registry belongs.
        required: True
        type: str
    registry_name:
        description:
            - The name of the container registry.
        required: True
        type: str
    webhook_name:
        description:
            - The name of the webhook.
        required: True
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - "Zim Kalinowski (@zikalino)"

'''

EXAMPLES = '''
- name: Get instance of Webhook
  azure_rm_containerregistrywebhook_info:
    resource_group: resource_group_name
    registry_name: registry_name
    webhook_name: webhook_name
'''

RETURN = '''
webhooks:
    description: A list of dict results where the key is the name of the Webhook and the values are the facts for that Webhook.
    returned: always
    type: complex
    contains:
        webhook_name:
            description: The key is the name of the server that the values relate to.
            type: complex
            contains:
                id:
                    description:
                        - The resource ID.
                    returned: always
                    type: str
                    sample: "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/myResourceGroup/providers/Microsoft.ContainerRegistry/registr
                            ies/myRegistry/webhooks/myWebhook"
                name:
                    description:
                        - The name of the resource.
                    returned: always
                    type: str
                    sample: myWebhook
                type:
                    description:
                        - The type of the resource.
                    returned: always
                    type: str
                    sample: Microsoft.ContainerRegistry/registries/webhooks
                location:
                    description:
                        - The location of the resource. This cannot be changed after the resource is created.
                    returned: always
                    type: str
                    sample: westus
                status:
                    description:
                        - "The status of the webhook at the time the operation was called. Possible values include: 'enabled', 'disabled'"
                    returned: always
                    type: str
                    sample: enabled
                scope:
                    description:
                        - "The scope of repositories where the event can be triggered. For example, 'foo:*' means events for all tags under repository 'foo'.
                           'foo:bar' means events for 'foo:bar' only. 'foo' is equivalent to 'foo:latest'. Empty means all events."
                    returned: always
                    type: str
                    sample: myRepository
                actions:
                    description:
                        - The list of actions that trigger the webhook to post notifications.
                    returned: always
                    type: str
                    sample: "[\n\n  'push'\n\n]"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMWebhooksFacts(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            registry_name=dict(
                type='str',
                required=True
            ),
            webhook_name=dict(
                type='str',
                required=True
            )
        )
        # store the results of the module operation
        self.results = dict(
            changed=False,
            ansible_facts=dict()
        )
        self.resource_group = None
        self.registry_name = None
        self.webhook_name = None
        super(AzureRMWebhooksFacts, self).__init__(self.module_arg_spec, supports_tags=False, supports_check_mode=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if (self.resource_group is not None and
                self.registry_name is not None and
                self.webhook_name is not None):
            self.results['webhooks'] = self.get()
        return self.results

    def get(self):
        '''
        Gets facts of the specified Webhook.

        :return: deserialized Webhookinstance state dictionary
        '''
        response = None
        results = {}
        try:
            response = self.containerregistry_client.webhooks.get(resource_group_name=self.resource_group,
                                                                  registry_name=self.registry_name,
                                                                  webhook_name=self.webhook_name)
            self.log("Response : {0}".format(response))
        except ResourceNotFoundError as e:
            self.log('Could not get facts for Webhooks: {0}'.format(str(e)))

        if response is not None:
            results[response.name] = response.as_dict()

        return results


def main():
    AzureRMWebhooksFacts()


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The type5_pw filter plugin
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
name: type5_pw
author: Ken Celenza (@itdependsnetworks)
version_added: "1.0.0"
short_description: The type5_pw filter plugin.
description:
  - Filter plugin to produce cisco type5 hashed password.
  - Using the parameters below - C(xml_data | ansible.netcommon.type5_pw(template.yml))
notes:
  - The filter plugin generates cisco type5 hashed password.
options:
  password:
    description:
    - The password to be hashed.
    type: str
    required: True
  salt:
    description:
    - Mention the salt to hash the password.
    type: str
"""

EXAMPLES = r"""
# Using type5_pw

- name: Set some facts
  ansible.builtin.set_fact:
    password: "cisco@123"

- name: Filter type5_pw invocation
  ansible.builtin.debug:
    msg: "{{ password | ansible.netcommon.type5_pw(salt='avs') }}"


# Task Output
# -----------
#
# TASK [Set some facts]
# ok: [host] => changed=false
#   ansible_facts:
#     password: cisco@123

# TASK [Filter type5_pw invocation]
# ok: [host] =>
#   msg: $1$avs$uSTOEMh65qzvpb9yBMpzd/
"""

from ansible.errors import AnsibleFilterError
from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)

from ansible_collections.ansible.netcommon.plugins.plugin_utils.type5_pw import type5_pw


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment


@pass_environment
def _type5_pw(*args, **kwargs):
    """Extend vlan data"""

    keys = ["password", "salt"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="type5_pw")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return type5_pw(**updated_data)


class FilterModule(object):
    """type5_pw"""

    def filters(self):
        """a mapping of filter names to functions"""
        return {"type5_pw": _type5_pw}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
from ansible.plugins.action import ActionBase
try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.dnac.plugins.plugin_utils.dnac import (
    DNACSDK,
    dnac_argument_spec,
    dnac_compare_equality,
    get_dict_result,
)
from ansible_collections.cisco.dnac.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = dnac_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present", "absent"]),
    ssid=dict(type="str"),
    authType=dict(type="str"),
    passphrase=dict(type="str"),
    isFastLaneEnabled=dict(type="bool"),
    isMacFilteringEnabled=dict(type="bool"),
    ssidRadioType=dict(type="str"),
    isBroadcastSSID=dict(type="bool"),
    fastTransition=dict(type="str"),
    sessionTimeOutEnable=dict(type="bool"),
    sessionTimeOut=dict(type="int"),
    clientExclusionEnable=dict(type="bool"),
    clientExclusionTimeout=dict(type="int"),
    basicServiceSetMaxIdleEnable=dict(type="bool"),
    basicServiceSetClientIdleTimeout=dict(type="int"),
    directedMulticastServiceEnable=dict(type="bool"),
    neighborListEnable=dict(type="bool"),
    managementFrameProtectionClientprotection=dict(type="str"),
    nasOptions=dict(type="list"),
    profileName=dict(type="str"),
    aaaOverride=dict(type="bool"),
    coverageHoleDetectionEnable=dict(type="bool"),
    protectedManagementFrame=dict(type="str"),
    multiPSKSettings=dict(type="list"),
    clientRateLimit=dict(type="int"),
    rsnCipherSuiteGcmp256=dict(type="bool"),
    rsnCipherSuiteCcmp256=dict(type="bool"),
    rsnCipherSuiteGcmp128=dict(type="bool"),
    rsnCipherSuiteCcmp128=dict(type="bool"),
    ghz6PolicyClientSteering=dict(type="bool"),
    isAuthKey8021x=dict(type="bool"),
    isAuthKey8021xPlusFT=dict(type="bool"),
    isAuthKey8021x_SHA256=dict(type="bool"),
    isAuthKeySae=dict(type="bool"),
    isAuthKeySaePlusFT=dict(type="bool"),
    isAuthKeyPSK=dict(type="bool"),
    isAuthKeyPSKPlusFT=dict(type="bool"),
    isAuthKeyOWE=dict(type="bool"),
    isAuthKeyEasyPSK=dict(type="bool"),
    isAuthKeyPSKSHA256=dict(type="bool"),
    openSsid=dict(type="str"),
    wlanBandSelectEnable=dict(type="bool"),
    isEnabled=dict(type="bool"),
    authServers=dict(type="list"),
    acctServers=dict(type="list"),
    egressQos=dict(type="str"),
    ingressQos=dict(type="str"),
    wlanType=dict(type="str"),
    l3AuthType=dict(type="str"),
    authServer=dict(type="str"),
    externalAuthIpAddress=dict(type="str"),
    webPassthrough=dict(type="bool"),
    sleepingClientEnable=dict(type="bool"),
    sleepingClientTimeout=dict(type="int"),
    aclName=dict(type="str"),
    isPosturingEnabled=dict(type="bool"),
    isAuthKeySuiteB1x=dict(type="bool"),
    isAuthKeySuiteB1921x=dict(type="bool"),
    isAuthKeySaeExt=dict(type="bool"),
    isAuthKeySaeExtPlusFT=dict(type="bool"),
    isApBeaconProtectionEnabled=dict(type="bool"),
    ghz24Policy=dict(type="str"),
    cckmTsfTolerance=dict(type="int"),
    isCckmEnabled=dict(type="bool"),
    isHex=dict(type="bool"),
    isRandomMacFilterEnabled=dict(type="bool"),
    fastTransitionOverTheDistributedSystemEnable=dict(type="bool"),
    siteId=dict(type="str"),
    id=dict(type="str"),
))

required_if = [
    ("state", "present", ["id", "siteId"], True),
    ("state", "absent", ["id", "siteId"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class SitesWirelessSettingsSsidsV1(object):
    def __init__(self, params, dnac):
        self.dnac = dnac
        self.new_object = dict(
            ssid=params.get("ssid"),
            authType=params.get("authType"),
            passphrase=params.get("passphrase"),
            isFastLaneEnabled=params.get("isFastLaneEnabled"),
            isMacFilteringEnabled=params.get("isMacFilteringEnabled"),
            ssidRadioType=params.get("ssidRadioType"),
            isBroadcastSSID=params.get("isBroadcastSSID"),
            fastTransition=params.get("fastTransition"),
            sessionTimeOutEnable=params.get("sessionTimeOutEnable"),
            sessionTimeOut=params.get("sessionTimeOut"),
            clientExclusionEnable=params.get("clientExclusionEnable"),
            clientExclusionTimeout=params.get("clientExclusionTimeout"),
            basicServiceSetMaxIdleEnable=params.get("basicServiceSetMaxIdleEnable"),
            basicServiceSetClientIdleTimeout=params.get("basicServiceSetClientIdleTimeout"),
            directedMulticastServiceEnable=params.get("directedMulticastServiceEnable"),
            neighborListEnable=params.get("neighborListEnable"),
            managementFrameProtectionClientprotection=params.get("managementFrameProtectionClientprotection"),
            nasOptions=params.get("nasOptions"),
            profileName=params.get("profileName"),
            aaaOverride=params.get("aaaOverride"),
            coverageHoleDetectionEnable=params.get("coverageHoleDetectionEnable"),
            protectedManagementFrame=params.get("protectedManagementFrame"),
            multiPSKSettings=params.get("multiPSKSettings"),
            clientRateLimit=params.get("clientRateLimit"),
            rsnCipherSuiteGcmp256=params.get("rsnCipherSuiteGcmp256"),
            rsnCipherSuiteCcmp256=params.get("rsnCipherSuiteCcmp256"),
            rsnCipherSuiteGcmp128=params.get("rsnCipherSuiteGcmp128"),
            rsnCipherSuiteCcmp128=params.get("rsnCipherSuiteCcmp128"),
            ghz6PolicyClientSteering=params.get("ghz6PolicyClientSteering"),
            isAuthKey8021x=params.get("isAuthKey8021x"),
            isAuthKey8021xPlusFT=params.get("isAuthKey8021xPlusFT"),
            isAuthKey8021x_SHA256=params.get("isAuthKey8021x_SHA256"),
            isAuthKeySae=params.get("isAuthKeySae"),
            isAuthKeySaePlusFT=params.get("isAuthKeySaePlusFT"),
            isAuthKeyPSK=params.get("isAuthKeyPSK"),
            isAuthKeyPSKPlusFT=params.get("isAuthKeyPSKPlusFT"),
            isAuthKeyOWE=params.get("isAuthKeyOWE"),
            isAuthKeyEasyPSK=params.get("isAuthKeyEasyPSK"),
            isAuthKeyPSKSHA256=params.get("isAuthKeyPSKSHA256"),
            openSsid=params.get("openSsid"),
            wlanBandSelectEnable=params.get("wlanBandSelectEnable"),
            isEnabled=params.get("isEnabled"),
            authServers=params.get("authServers"),
            acctServers=params.get("acctServers"),
            egressQos=params.get("egressQos"),
            ingressQos=params.get("ingressQos"),
            wlanType=params.get("wlanType"),
            l3AuthType=params.get("l3AuthType"),
            authServer=params.get("authServer"),
            externalAuthIpAddress=params.get("externalAuthIpAddress"),
            webPassthrough=params.get("webPassthrough"),
            sleepingClientEnable=params.get("sleepingClientEnable"),
            sleepingClientTimeout=params.get("sleepingClientTimeout"),
            aclName=params.get("aclName"),
            isPosturingEnabled=params.get("isPosturingEnabled"),
            isAuthKeySuiteB1x=params.get("isAuthKeySuiteB1x"),
            isAuthKeySuiteB1921x=params.get("isAuthKeySuiteB1921x"),
            isAuthKeySaeExt=params.get("isAuthKeySaeExt"),
            isAuthKeySaeExtPlusFT=params.get("isAuthKeySaeExtPlusFT"),
            isApBeaconProtectionEnabled=params.get("isApBeaconProtectionEnabled"),
            ghz24Policy=params.get("ghz24Policy"),
            cckmTsfTolerance=params.get("cckmTsfTolerance"),
            isCckmEnabled=params.get("isCckmEnabled"),
            isHex=params.get("isHex"),
            isRandomMacFilterEnabled=params.get("isRandomMacFilterEnabled"),
            fastTransitionOverTheDistributedSystemEnable=params.get("fastTransitionOverTheDistributedSystemEnable"),
            site_id=params.get("siteId"),
            id=params.get("id"),
        )

    def get_all_params(self, name=None, id=None):
        new_object_params = {}
        new_object_params['limit'] = self.new_object.get('limit')
        new_object_params['offset'] = self.new_object.get('offset')
        new_object_params['site_id'] = self.new_object.get('siteId') or \
            self.new_object.get('site_id')
        return new_object_params

    def create_params(self):
        new_object_params = {}
        new_object_params['ssid'] = self.new_object.get('ssid')
        new_object_params['authType'] = self.new_object.get('authType')
        new_object_params['passphrase'] = self.new_object.get('passphrase')
        new_object_params['isFastLaneEnabled'] = self.new_object.get('isFastLaneEnabled')
        new_object_params['isMacFilteringEnabled'] = self.new_object.get('isMacFilteringEnabled')
        new_object_params['ssidRadioType'] = self.new_object.get('ssidRadioType')
        new_object_params['isBroadcastSSID'] = self.new_object.get('isBroadcastSSID')
        new_object_params['fastTransition'] = self.new_object.get('fastTransition')
        new_object_params['sessionTimeOutEnable'] = self.new_object.get('sessionTimeOutEnable')
        new_object_params['sessionTimeOut'] = self.new_object.get('sessionTimeOut')
        new_object_params['clientExclusionEnable'] = self.new_object.get('clientExclusionEnable')
        new_object_params['clientExclusionTimeout'] = self.new_object.get('clientExclusionTimeout')
        new_object_params['basicServiceSetMaxIdleEnable'] = self.new_object.get('basicServiceSetMaxIdleEnable')
        new_object_params['basicServiceSetClientIdleTimeout'] = self.new_object.get('basicServiceSetClientIdleTimeout')
        new_object_params['directedMulticastServiceEnable'] = self.new_object.get('directedMulticastServiceEnable')
        new_object_params['neighborListEnable'] = self.new_object.get('neighborListEnable')
        new_object_params['managementFrameProtectionClientprotection'] = self.new_object.get('managementFrameProtectionClientprotection')
        new_object_params['nasOptions'] = self.new_object.get('nasOptions')
        new_object_params['profileName'] = self.new_object.get('profileName')
        new_object_params['aaaOverride'] = self.new_object.get('aaaOverride')
        new_object_params['coverageHoleDetectionEnable'] = self.new_object.get('coverageHoleDetectionEnable')
        new_object_params['protectedManagementFrame'] = self.new_object.get('protectedManagementFrame')
        new_object_params['multiPSKSettings'] = self.new_object.get('multiPSKSettings')
        new_object_params['clientRateLimit'] = self.new_object.get('clientRateLimit')
        new_object_params['rsnCipherSuiteGcmp256'] = self.new_object.get('rsnCipherSuiteGcmp256')
        new_object_params['rsnCipherSuiteCcmp256'] = self.new_object.get('rsnCipherSuiteCcmp256')
        new_object_params['rsnCipherSuiteGcmp128'] = self.new_object.get('rsnCipherSuiteGcmp128')
        new_object_params['rsnCipherSuiteCcmp128'] = self.new_object.get('rsnCipherSuiteCcmp128')
        new_object_params['ghz6PolicyClientSteering'] = self.new_object.get('ghz6PolicyClientSteering')
        new_object_params['isAuthKey8021x'] = self.new_object.get('isAuthKey8021x')
        new_object_params['isAuthKey8021xPlusFT'] = self.new_object.get('isAuthKey8021xPlusFT')
        new_object_params['isAuthKey8021x_SHA256'] = self.new_object.get('isAuthKey8021x_SHA256')
        new_object_params['isAuthKeySae'] = self.new_object.get('isAuthKeySae')
        new_object_params['isAuthKeySaePlusFT'] = self.new_object.get('isAuthKeySaePlusFT')
        new_object_params['isAuthKeyPSK'] = self.new_object.get('isAuthKeyPSK')
        new_object_params['isAuthKeyPSKPlusFT'] = self.new_object.get('isAuthKeyPSKPlusFT')
        new_object_params['isAuthKeyOWE'] = self.new_object.get('isAuthKeyOWE')
        new_object_params['isAuthKeyEasyPSK'] = self.new_object.get('isAuthKeyEasyPSK')
        new_object_params['isAuthKeyPSKSHA256'] = self.new_object.get('isAuthKeyPSKSHA256')
        new_object_params['openSsid'] = self.new_object.get('openSsid')
        new_object_params['wlanBandSelectEnable'] = self.new_object.get('wlanBandSelectEnable')
        new_object_params['isEnabled'] = self.new_object.get('isEnabled')
        new_object_params['authServers'] = self.new_object.get('authServers')
        new_object_params['acctServers'] = self.new_object.get('acctServers')
        new_object_params['egressQos'] = self.new_object.get('egressQos')
        new_object_params['ingressQos'] = self.new_object.get('ingressQos')
        new_object_params['wlanType'] = self.new_object.get('wlanType')
        new_object_params['l3AuthType'] = self.new_object.get('l3AuthType')
        new_object_params['authServer'] = self.new_object.get('authServer')
        new_object_params['externalAuthIpAddress'] = self.new_object.get('externalAuthIpAddress')
        new_object_params['webPassthrough'] = self.new_object.get('webPassthrough')
        new_object_params['sleepingClientEnable'] = self.new_object.get('sleepingClientEnable')
        new_object_params['sleepingClientTimeout'] = self.new_object.get('sleepingClientTimeout')
        new_object_params['aclName'] = self.new_object.get('aclName')
        new_object_params['isPosturingEnabled'] = self.new_object.get('isPosturingEnabled')
        new_object_params['isAuthKeySuiteB1x'] = self.new_object.get('isAuthKeySuiteB1x')
        new_object_params['isAuthKeySuiteB1921x'] = self.new_object.get('isAuthKeySuiteB1921x')
        new_object_params['isAuthKeySaeExt'] = self.new_object.get('isAuthKeySaeExt')
        new_object_params['isAuthKeySaeExtPlusFT'] = self.new_object.get('isAuthKeySaeExtPlusFT')
        new_object_params['isApBeaconProtectionEnabled'] = self.new_object.get('isApBeaconProtectionEnabled')
        new_object_params['ghz24Policy'] = self.new_object.get('ghz24Policy')
        new_object_params['cckmTsfTolerance'] = self.new_object.get('cckmTsfTolerance')
        new_object_params['isCckmEnabled'] = self.new_object.get('isCckmEnabled')
        new_object_params['isHex'] = self.new_object.get('isHex')
        new_object_params['isRandomMacFilterEnabled'] = self.new_object.get('isRandomMacFilterEnabled')
        new_object_params['fastTransitionOverTheDistributedSystemEnable'] = self.new_object.get('fastTransitionOverTheDistributedSystemEnable')
        new_object_params['siteId'] = self.new_object.get('siteId')
        return new_object_params

    def delete_by_id_params(self):
        new_object_params = {}
        new_object_params['site_id'] = self.new_object.get('site_id')
        new_object_params['id'] = self.new_object.get('id')
        return new_object_params

    def update_by_id_params(self):
        new_object_params = {}
        new_object_params['ssid'] = self.new_object.get('ssid')
        new_object_params['authType'] = self.new_object.get('authType')
        new_object_params['passphrase'] = self.new_object.get('passphrase')
        new_object_params['isFastLaneEnabled'] = self.new_object.get('isFastLaneEnabled')
        new_object_params['isMacFilteringEnabled'] = self.new_object.get('isMacFilteringEnabled')
        new_object_params['ssidRadioType'] = self.new_object.get('ssidRadioType')
        new_object_params['isBroadcastSSID'] = self.new_object.get('isBroadcastSSID')
        new_object_params['fastTransition'] = self.new_object.get('fastTransition')
        new_object_params['sessionTimeOutEnable'] = self.new_object.get('sessionTimeOutEnable')
        new_object_params['sessionTimeOut'] = self.new_object.get('sessionTimeOut')
        new_object_params['clientExclusionEnable'] = self.new_object.get('clientExclusionEnable')
        new_object_params['clientExclusionTimeout'] = self.new_object.get('clientExclusionTimeout')
        new_object_params['basicServiceSetMaxIdleEnable'] = self.new_object.get('basicServiceSetMaxIdleEnable')
        new_object_params['basicServiceSetClientIdleTimeout'] = self.new_object.get('basicServiceSetClientIdleTimeout')
        new_object_params['directedMulticastServiceEnable'] = self.new_object.get('directedMulticastServiceEnable')
        new_object_params['neighborListEnable'] = self.new_object.get('neighborListEnable')
        new_object_params['managementFrameProtectionClientprotection'] = self.new_object.get('managementFrameProtectionClientprotection')
        new_object_params['nasOptions'] = self.new_object.get('nasOptions')
        new_object_params['profileName'] = self.new_object.get('profileName')
        new_object_params['aaaOverride'] = self.new_object.get('aaaOverride')
        new_object_params['coverageHoleDetectionEnable'] = self.new_object.get('coverageHoleDetectionEnable')
        new_object_params['protectedManagementFrame'] = self.new_object.get('protectedManagementFrame')
        new_object_params['multiPSKSettings'] = self.new_object.get('multiPSKSettings')
        new_object_params['clientRateLimit'] = self.new_object.get('clientRateLimit')
        new_object_params['rsnCipherSuiteGcmp256'] = self.new_object.get('rsnCipherSuiteGcmp256')
        new_object_params['rsnCipherSuiteCcmp256'] = self.new_object.get('rsnCipherSuiteCcmp256')
        new_object_params['rsnCipherSuiteGcmp128'] = self.new_object.get('rsnCipherSuiteGcmp128')
        new_object_params['rsnCipherSuiteCcmp128'] = self.new_object.get('rsnCipherSuiteCcmp128')
        new_object_params['ghz6PolicyClientSteering'] = self.new_object.get('ghz6PolicyClientSteering')
        new_object_params['isAuthKey8021x'] = self.new_object.get('isAuthKey8021x')
        new_object_params['isAuthKey8021xPlusFT'] = self.new_object.get('isAuthKey8021xPlusFT')
        new_object_params['isAuthKey8021x_SHA256'] = self.new_object.get('isAuthKey8021x_SHA256')
        new_object_params['isAuthKeySae'] = self.new_object.get('isAuthKeySae')
        new_object_params['isAuthKeySaePlusFT'] = self.new_object.get('isAuthKeySaePlusFT')
        new_object_params['isAuthKeyPSK'] = self.new_object.get('isAuthKeyPSK')
        new_object_params['isAuthKeyPSKPlusFT'] = self.new_object.get('isAuthKeyPSKPlusFT')
        new_object_params['isAuthKeyOWE'] = self.new_object.get('isAuthKeyOWE')
        new_object_params['isAuthKeyEasyPSK'] = self.new_object.get('isAuthKeyEasyPSK')
        new_object_params['isAuthKeyPSKSHA256'] = self.new_object.get('isAuthKeyPSKSHA256')
        new_object_params['openSsid'] = self.new_object.get('openSsid')
        new_object_params['wlanBandSelectEnable'] = self.new_object.get('wlanBandSelectEnable')
        new_object_params['isEnabled'] = self.new_object.get('isEnabled')
        new_object_params['authServers'] = self.new_object.get('authServers')
        new_object_params['acctServers'] = self.new_object.get('acctServers')
        new_object_params['egressQos'] = self.new_object.get('egressQos')
        new_object_params['ingressQos'] = self.new_object.get('ingressQos')
        new_object_params['wlanType'] = self.new_object.get('wlanType')
        new_object_params['l3AuthType'] = self.new_object.get('l3AuthType')
        new_object_params['authServer'] = self.new_object.get('authServer')
        new_object_params['externalAuthIpAddress'] = self.new_object.get('externalAuthIpAddress')
        new_object_params['webPassthrough'] = self.new_object.get('webPassthrough')
        new_object_params['sleepingClientEnable'] = self.new_object.get('sleepingClientEnable')
        new_object_params['sleepingClientTimeout'] = self.new_object.get('sleepingClientTimeout')
        new_object_params['aclName'] = self.new_object.get('aclName')
        new_object_params['isPosturingEnabled'] = self.new_object.get('isPosturingEnabled')
        new_object_params['isAuthKeySuiteB1x'] = self.new_object.get('isAuthKeySuiteB1x')
        new_object_params['isAuthKeySuiteB1921x'] = self.new_object.get('isAuthKeySuiteB1921x')
        new_object_params['isAuthKeySaeExt'] = self.new_object.get('isAuthKeySaeExt')
        new_object_params['isAuthKeySaeExtPlusFT'] = self.new_object.get('isAuthKeySaeExtPlusFT')
        new_object_params['isApBeaconProtectionEnabled'] = self.new_object.get('isApBeaconProtectionEnabled')
        new_object_params['ghz24Policy'] = self.new_object.get('ghz24Policy')
        new_object_params['cckmTsfTolerance'] = self.new_object.get('cckmTsfTolerance')
        new_object_params['isCckmEnabled'] = self.new_object.get('isCckmEnabled')
        new_object_params['isHex'] = self.new_object.get('isHex')
        new_object_params['isRandomMacFilterEnabled'] = self.new_object.get('isRandomMacFilterEnabled')
        new_object_params['fastTransitionOverTheDistributedSystemEnable'] = self.new_object.get('fastTransitionOverTheDistributedSystemEnable')
        new_object_params['siteId'] = self.new_object.get('siteId')
        new_object_params['id'] = self.new_object.get('id')
        return new_object_params

    def get_object_by_name(self, name):
        result = None
        # NOTE: Does not have a get by name method or it is in another action
        try:
            items = self.dnac.exec(
                family="wireless",
                function="get_ssid_by_site_v1",
                params=self.get_all_params(name=name),
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'name', name)
        except Exception:
            result = None
        return result

    def get_object_by_id(self, id):
        result = None
        try:
            items = self.dnac.exec(
                family="wireless",
                function="get_ssid_by_id_v1",
                params={"id": id}
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'id', id)
        except Exception:
            result = None
        return result

    def exists(self):
        id_exists = False
        name_exists = False
        prev_obj = None
        o_id = self.new_object.get("id")
        name = self.new_object.get("name")
        if o_id:
            prev_obj = self.get_object_by_id(o_id)
            id_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if not id_exists and name:
            prev_obj = self.get_object_by_name(name)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if name_exists:
            _id = prev_obj.get("id")
            if id_exists and name_exists and o_id != _id:
                raise InconsistentParameters("The 'id' and 'name' params don't refer to the same object")
            if _id:
                self.new_object.update(dict(id=_id))
            if _id:
                prev_obj = self.get_object_by_id(_id)
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("ssid", "ssid"),
            ("authType", "authType"),
            ("passphrase", "passphrase"),
            ("isFastLaneEnabled", "isFastLaneEnabled"),
            ("isMacFilteringEnabled", "isMacFilteringEnabled"),
            ("ssidRadioType", "ssidRadioType"),
            ("isBroadcastSSID", "isBroadcastSSID"),
            ("fastTransition", "fastTransition"),
            ("sessionTimeOutEnable", "sessionTimeOutEnable"),
            ("sessionTimeOut", "sessionTimeOut"),
            ("clientExclusionEnable", "clientExclusionEnable"),
            ("clientExclusionTimeout", "clientExclusionTimeout"),
            ("basicServiceSetMaxIdleEnable", "basicServiceSetMaxIdleEnable"),
            ("basicServiceSetClientIdleTimeout", "basicServiceSetClientIdleTimeout"),
            ("directedMulticastServiceEnable", "directedMulticastServiceEnable"),
            ("neighborListEnable", "neighborListEnable"),
            ("managementFrameProtectionClientprotection", "managementFrameProtectionClientprotection"),
            ("nasOptions", "nasOptions"),
            ("profileName", "profileName"),
            ("aaaOverride", "aaaOverride"),
            ("coverageHoleDetectionEnable", "coverageHoleDetectionEnable"),
            ("protectedManagementFrame", "protectedManagementFrame"),
            ("multiPSKSettings", "multiPSKSettings"),
            ("clientRateLimit", "clientRateLimit"),
            ("rsnCipherSuiteGcmp256", "rsnCipherSuiteGcmp256"),
            ("rsnCipherSuiteCcmp256", "rsnCipherSuiteCcmp256"),
            ("rsnCipherSuiteGcmp128", "rsnCipherSuiteGcmp128"),
            ("rsnCipherSuiteCcmp128", "rsnCipherSuiteCcmp128"),
            ("ghz6PolicyClientSteering", "ghz6PolicyClientSteering"),
            ("isAuthKey8021x", "isAuthKey8021x"),
            ("isAuthKey8021xPlusFT", "isAuthKey8021xPlusFT"),
            ("isAuthKey8021x_SHA256", "isAuthKey8021x_SHA256"),
            ("isAuthKeySae", "isAuthKeySae"),
            ("isAuthKeySaePlusFT", "isAuthKeySaePlusFT"),
            ("isAuthKeyPSK", "isAuthKeyPSK"),
            ("isAuthKeyPSKPlusFT", "isAuthKeyPSKPlusFT"),
            ("isAuthKeyOWE", "isAuthKeyOWE"),
            ("isAuthKeyEasyPSK", "isAuthKeyEasyPSK"),
            ("isAuthKeyPSKSHA256", "isAuthKeyPSKSHA256"),
            ("openSsid", "openSsid"),
            ("wlanBandSelectEnable", "wlanBandSelectEnable"),
            ("isEnabled", "isEnabled"),
            ("authServers", "authServers"),
            ("acctServers", "acctServers"),
            ("egressQos", "egressQos"),
            ("ingressQos", "ingressQos"),
            ("wlanType", "wlanType"),
            ("l3AuthType", "l3AuthType"),
            ("authServer", "authServer"),
            ("externalAuthIpAddress", "externalAuthIpAddress"),
            ("webPassthrough", "webPassthrough"),
            ("sleepingClientEnable", "sleepingClientEnable"),
            ("sleepingClientTimeout", "sleepingClientTimeout"),
            ("aclName", "aclName"),
            ("isPosturingEnabled", "isPosturingEnabled"),
            ("isAuthKeySuiteB1x", "isAuthKeySuiteB1x"),
            ("isAuthKeySuiteB1921x", "isAuthKeySuiteB1921x"),
            ("isAuthKeySaeExt", "isAuthKeySaeExt"),
            ("isAuthKeySaeExtPlusFT", "isAuthKeySaeExtPlusFT"),
            ("isApBeaconProtectionEnabled", "isApBeaconProtectionEnabled"),
            ("ghz24Policy", "ghz24Policy"),
            ("cckmTsfTolerance", "cckmTsfTolerance"),
            ("isCckmEnabled", "isCckmEnabled"),
            ("isHex", "isHex"),
            ("isRandomMacFilterEnabled", "isRandomMacFilterEnabled"),
            ("fastTransitionOverTheDistributedSystemEnable", "fastTransitionOverTheDistributedSystemEnable"),
            ("siteId", "site_id"),
            ("id", "id"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (DNAC) params
        # If any does not have eq params, it requires update
        return any(not dnac_compare_equality(current_obj.get(dnac_param),
                                             requested_obj.get(ansible_param))
                   for (dnac_param, ansible_param) in obj_params)

    def create(self):
        result = self.dnac.exec(
            family="wireless",
            function="create_ssid_v1",
            params=self.create_params(),
            op_modifies=True,
        )
        return result

    def update(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        result = None
        if not id:
            prev_obj_name = self.get_object_by_name(name)
            id_ = None
            if prev_obj_name:
                id_ = prev_obj_name.get("id")
            if id_:
                self.new_object.update(dict(id=id_))
        result = self.dnac.exec(
            family="wireless",
            function="update_ssid_v1",
            params=self.update_by_id_params(),
            op_modifies=True,
        )
        return result

    def delete(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        result = None
        if not id:
            prev_obj_name = self.get_object_by_name(name)
            id_ = None
            if prev_obj_name:
                id_ = prev_obj_name.get("id")
            if id_:
                self.new_object.update(dict(id=id_))
        result = self.dnac.exec(
            family="wireless",
            function="delete_ssid_v1",
            params=self.delete_by_id_params(),
        )
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail("ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        dnac = DNACSDK(self._task.args)
        obj = SitesWirelessSettingsSsidsV1(self._task.args, dnac)

        state = self._task.args.get("state")

        response = None

        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    response = obj.update()
                    dnac.object_updated()
                else:
                    response = prev_obj
                    dnac.object_already_present()
            else:
                response = obj.create()
                dnac.object_created()

        elif state == "absent":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                response = obj.delete()
                dnac.object_deleted()
            else:
                dnac.object_already_absent()

        self._result.update(dict(dnac_response=response))
        self._result.update(dnac.exit_json())
        return self._result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: user_equipment_subscriber_info
short_description: Information module for User Equipment Subscriber Info
description:
- Get User Equipment Subscriber Info by id.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  subscriberId:
    description:
    - SubscriberId path parameter. Unique ID for a subscriber object.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    user_equipment.UserEquipment.get_user_equipments_by_subscriber_id,

  - Paths used are
    get /api/v1/fiveg/user-equipment/subscriber/{subscriberId},

"""

EXAMPLES = r"""
- name: Get User Equipment Subscriber Info by id
  cisco.ise.user_equipment_subscriber_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    subscriberId: 7c9484cf-0ebc-47ad-a9ef-bc12729ed73d
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "description": "string",
        "deviceGroup": "string",
        "imei": "string",
        "createTime": "string",
        "updateTime": "string",
        "id": "string",
        "link": {
          "rel": "string",
          "href": "string",
          "type": "string"
        }
      }
    ]
"""



#!/usr/bin/python
#
# Copyright (c) 2021 Cole Neubauer, (@coleneubauer), xuzhang3 (@xuzhang3)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
module: azure_rm_adgroup_info
version_added: "1.6.0"
short_description: Get Azure Active Directory group info
description:
    - Get Azure Active Directory group info.
options:
    object_id:
        description:
            - The object id for the ad group.
            - returns the group which has this object ID.
        type: str
    attribute_name:
        description:
            - The name of an attribute that you want to match to I(attribute_value).
            - If I(attribute_name) is not a collection type it will return groups where I(attribute_name) is equal to I(attribute_value).
            - If I(attribute_name) is a collection type it will return groups where I(attribute_value) is in I(attribute_name).
        type: str
    attribute_value:
        description:
            - The value to match attribute_name to.
            - If I(attribute_name) is not a collection type it will return groups where I(attribute_name) is equal to I(attribute_value).
            - If I(attribute_name) is a collection type it will groups users where I(attribute_value) is in I(attribute_name).
        type: str
    odata_filter:
        description:
            - returns groups based on the the OData filter passed into this parameter.
        type: str
    check_membership:
        description:
            - The object ID of the contact, group, user, or service principal to check for membership against returned groups.
        type: str
    return_owners:
        description:
            - Indicate whether the owners of a group should be returned with the returned groups.
        default: False
        type: bool
    return_group_members:
        description:
            - Indicate whether the members of a group should be returned with the returned groups.
        default: False
        type: bool
    return_member_groups:
        description:
            - Indicate whether the groups in which a groups is a member should be returned with the returned groups.
        default: False
        type: bool
    raw_membership:
        description:
            - By default the group_members return property is flattened and partially filtered of non-User objects before return.\
              This argument disables those transformations.
        default: false
        type: bool
    all:
        description:
            - If True, will return all groups in tenant.
            - If False will return no users.
            - It is recommended that you instead identify a subset of groups and use filter.
        default: False
        type: bool
extends_documentation_fragment:
    - azure.azcollection.azure
author:
    - Cole Neubauer(@coleneubauer)
    - Xu Zhang(@xuzhang)
'''

EXAMPLES = '''
- name: Return a specific group using object_id
  azure_rm_adgroup_info:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx

- name: Return a specific group using object_id and  return the owners of the group
  azure_rm_adgroup_info:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    return_owners: true

- name: Return a specific group using object_id and return the owners and members of the group
  azure_rm_adgroup_info:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    return_owners: true
    return_group_members: true
- name: Return a specific group using object_id and return the owners and members of the group. Return service principals and nested groups.
  azure_rm_adgroup_info:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    return_owners: true
    return_group_members: true
    raw_membership: true

- name: Return a specific group using object_id and return the groups the group is a member of
  azure_rm_adgroup_info:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    return_member_groups: true

- name: Return a specific group using object_id and check an ID for membership
  azure_rm_adgroup_info:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    check_membership: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx

- name: Return a specific group using displayName for attribute_name
  azure_rm_adgroup_info:
    attribute_name: "displayName"
    attribute_value: "Display-Name-Of-AD-Group"

- name: Return groups matching odata_filter
  azure_rm_adgroup_info:
    odata_filter: "mailNickname eq 'Mail-Nickname-Of-AD-Group'"

- name: Return all groups
  azure_rm_adgroup_info:
    all: true
'''

RETURN = '''
object_id:
    description:
        - The object_id for the group.
    type: str
    returned: always
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
display_name:
    description:
        - The display name of the group.
    returned: always
    type: str
    sample: GroupName
mail_nickname:
    description:
        - The mail alias for the group.
    returned: always
    type: str
    sample: groupname
mail_enabled:
    description:
        - Whether the group is mail-enabled. Must be false. This is because only pure security groups can be created using the Graph API.
    returned: always
    type: bool
    sample: False
security_enabled:
    description:
        - Whether the group is security-enable.
    returned: always
    type: bool
    sample: False
mail:
    description:
        - The primary email address of the group.
    returned: always
    type: str
    sample: group@contoso.com
group_owners:
    description:
        - The owners of the group.
    returned: always
    type: list
group_members:
    description:
        - The members of the group. If raw_membership is set, this field may contain non-user objects (groups, service principals, etc)
    returned: always
    type: list
description:
    description:
        - An optional description for the group.
    type: str
    returned: always
    sample: 'fortest'
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBase

try:
    import asyncio
    from msgraph.generated.groups.groups_request_builder import GroupsRequestBuilder
    from msgraph.generated.groups.item.transitive_members.transitive_members_request_builder import \
        TransitiveMembersRequestBuilder
    from msgraph.generated.groups.item.get_member_groups.get_member_groups_post_request_body import \
        GetMemberGroupsPostRequestBody
    from msgraph.generated.groups.item.group_item_request_builder import GroupItemRequestBuilder
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMADGroupInfo(AzureRMModuleBase):
    def __init__(self):

        self.module_arg_spec = dict(
            object_id=dict(type='str'),
            attribute_name=dict(type='str'),
            attribute_value=dict(type='str'),
            odata_filter=dict(type='str'),
            check_membership=dict(type='str'),
            return_owners=dict(type='bool', default=False),
            return_group_members=dict(type='bool', default=False),
            return_member_groups=dict(type='bool', default=False),
            raw_membership=dict(type='bool', default=False),
            all=dict(type='bool', default=False),
        )

        self.object_id = None
        self.attribute_name = None
        self.attribute_value = None
        self.odata_filter = None
        self.check_membership = None
        self.return_owners = False
        self.return_group_members = False
        self.return_member_groups = False
        self.raw_membership = False
        self.all = False

        self.results = dict(changed=False)
        self._client = None

        mutually_exclusive = [['odata_filter', 'attribute_name', 'object_id', 'all']]
        required_together = [['attribute_name', 'attribute_value']]
        required_one_of = [['odata_filter', 'attribute_name', 'object_id', 'all']]

        super(AzureRMADGroupInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 supports_tags=False,
                                                 mutually_exclusive=mutually_exclusive,
                                                 required_together=required_together,
                                                 required_one_of=required_one_of,
                                                 is_ad_resource=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        ad_groups = []

        try:
            self._client = self.get_msgraph_client()

            if self.object_id is not None:
                ad_groups = [asyncio.get_event_loop().run_until_complete(self.get_group(self.object_id))]
            elif self.attribute_name is not None and self.attribute_value is not None:
                ad_groups = asyncio.get_event_loop().run_until_complete(
                    self.get_group_list(filter="{0} eq '{1}'".format(self.attribute_name, self.attribute_value)))
            elif self.odata_filter is not None:  # run a filter based on user input
                ad_groups = asyncio.get_event_loop().run_until_complete(self.get_group_list(filter=self.odata_filter))
            elif self.all:
                ad_groups = asyncio.get_event_loop().run_until_complete(self.get_group_list())
            self.results['ad_groups'] = [self.set_results(group) for group in ad_groups]
        except Exception as e:
            self.fail("failed to get ad group info {0}".format(str(e)))

        return self.results

    def application_to_dict(self, object):
        return dict(
            app_id=object.app_id,
            object_id=object.object_id,
            display_name=object.display_name,
        )

    def serviceprincipal_to_dict(self, object):
        return dict(
            app_id=object.app_id,
            object_id=object.id,
            app_display_name=object.display_name,
            app_role_assignment_required=object.app_role_assignment_required
        )

    def group_to_dict(self, object):
        return dict(
            object_id=object.id,
            display_name=object.display_name,
            mail_nickname=object.mail_nickname,
            mail_enabled=object.mail_enabled,
            security_enabled=object.security_enabled,
            mail=object.mail,
            description=object.description
        )

    def user_to_dict(self, object):
        return dict(
            object_id=object.id,
            display_name=object.display_name,
            user_principal_name=object.user_principal_name,
            mail_nickname=object.mail_nickname,
            mail=object.mail,
            account_enabled=object.account_enabled,
            user_type=object.user_type
        )

    def result_to_dict(self, object):
        if object.odata_type == "#microsoft.graph.group":
            return self.group_to_dict(object)
        elif object.odata_type == "#microsoft.graph.user":
            return self.user_to_dict(object)
        elif object.odata_type == "#microsoft.graph.application":
            return self.application_to_dict(object)
        elif object.odata_type == "#microsoft.graph.servicePrincipal":
            return self.serviceprincipal_to_dict(object)
        else:
            return object.odata_type

    def set_results(self, object):
        results = self.group_to_dict(object)

        if results["object_id"] and self.return_owners:
            ret = asyncio.get_event_loop().run_until_complete(self.get_group_owners(results["object_id"]))
            results["group_owners"] = [self.result_to_dict(object) for object in ret.value]

        if results["object_id"] and self.return_group_members:
            ret = asyncio.get_event_loop().run_until_complete(self.get_group_members(results["object_id"]))
            results["group_members"] = [self.result_to_dict(object) for object in ret]

        if results["object_id"] and self.return_member_groups:
            ret = asyncio.get_event_loop().run_until_complete(self.get_member_groups(results["object_id"]))
            results["member_groups"] = [self.result_to_dict(object) for object in list(ret.value)]

        if results["object_id"] and self.check_membership:
            filter = "id eq '{0}' ".format(self.check_membership)
            ret = asyncio.get_event_loop().run_until_complete(self.get_group_members(results["object_id"], filter))
            results["is_member_of"] = True if ret and len(ret) != 0 else False

        return results

    async def get_group(self, group_id):
        return await self._client.groups.by_group_id(group_id).get()

    async def get_group_list(self, filter=None):
        kwargs = {}
        if filter:
            request_configuration = GroupsRequestBuilder.GroupsRequestBuilderGetRequestConfiguration(
                query_parameters=GroupsRequestBuilder.GroupsRequestBuilderGetQueryParameters(
                    count=True,
                    filter=filter,
                ),
            )
            kwargs["request_configuration"] = request_configuration

        groups = []
        # paginated response can be quite large
        response = await self._client.groups.get(**kwargs)
        if response:
            groups += response.value
        while response is not None and response.odata_next_link is not None:
            response = await self._client.groups.with_url(response.odata_next_link).get(**kwargs)
            if response:
                groups += response.value

        return groups

    async def get_group_owners(self, group_id):
        request_configuration = GroupsRequestBuilder.GroupsRequestBuilderGetRequestConfiguration(
            query_parameters=GroupsRequestBuilder.GroupsRequestBuilderGetQueryParameters(
                count=True,
                select=['id', 'displayName', 'userPrincipalName', 'mailNickname', 'mail', 'accountEnabled', 'userType',
                        'appId', 'appRoleAssignmentRequired']

            ),
        )
        return await self._client.groups.by_group_id(group_id).owners.get(request_configuration=request_configuration)

    async def get_group_members(self, group_id, filters=None):
        if self.raw_membership:
            return await self.get_raw_group_members(group_id, filters)
        else:
            return await self.get_transitive_group_members(group_id, filters)

    async def get_transitive_group_members(self, group_id, filters=None):
        request_configuration = TransitiveMembersRequestBuilder.TransitiveMembersRequestBuilderGetRequestConfiguration(
            query_parameters=TransitiveMembersRequestBuilder.TransitiveMembersRequestBuilderGetQueryParameters(
                count=True,
            ),
        )
        if filters:
            request_configuration.query_parameters.filter = filters
        try:
            response = await self._client.groups.by_group_id(group_id).transitive_members.get(
                request_configuration=request_configuration)
            return response.value
        except Exception:
            return

    async def get_raw_group_members(self, group_id, filters=None):
        request_configuration = GroupItemRequestBuilder.GroupItemRequestBuilderGetRequestConfiguration(
            query_parameters=GroupItemRequestBuilder.GroupItemRequestBuilderGetQueryParameters(
                # this ensures service principals are returned
                # see https://learn.microsoft.com/en-us/graph/api/group-list-members?view=graph-rest-1.0&tabs=http
                expand=["members"]
            ),
        )
        if filters:
            request_configuration.query_parameters.filter = filters
        return await self._client.groups.by_group_id(group_id).members.get(
            request_configuration=request_configuration)

    async def get_member_groups(self, obj_id):
        request_body = GetMemberGroupsPostRequestBody(security_enabled_only=False)
        return await self._client.groups.by_group_id(obj_id).get_member_groups.post(body=request_body)


def main():
    AzureRMADGroupInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: app_policy_info
short_description: Information module for App Policy Info
description:
- This module represents an alias of the module app_policy_v1_info
version_added: '4.0.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  policyScope:
    description:
    - PolicyScope query parameter. Policy scope name.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Application Policy GetApplicationPolicyV1
  description: Complete reference of the GetApplicationPolicyV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-application-policy
notes:
  - SDK Method used are
    application_policy.ApplicationPolicy.get_application_policy_v1,

  - Paths used are
    get /dna/intent/api/v1/app-policy,
  - It should be noted that this module is an alias of app_policy_v1_info

"""

EXAMPLES = r"""
- name: Get all App Policy Info
  cisco.dnac.app_policy_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    policyScope: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "instanceId": 0,
          "displayName": "string",
          "instanceCreatedOn": 0,
          "instanceUpdatedOn": 0,
          "instanceVersion": 0,
          "createTime": 0,
          "deployed": true,
          "isSeeded": true,
          "isStale": true,
          "lastUpdateTime": 0,
          "name": "string",
          "namespace": "string",
          "provisioningState": "string",
          "qualifier": "string",
          "resourceVersion": 0,
          "targetIdList": [
            {}
          ],
          "type": "string",
          "cfsChangeInfo": [
            {}
          ],
          "customProvisions": [
            {}
          ],
          "deletePolicyStatus": "string",
          "internal": true,
          "isDeleted": true,
          "isEnabled": true,
          "isScopeStale": true,
          "iseReserved": true,
          "policyScope": "string",
          "policyStatus": "string",
          "priority": 0,
          "pushed": true,
          "advancedPolicyScope": {
            "id": "string",
            "instanceId": 0,
            "displayName": "string",
            "instanceCreatedOn": 0,
            "instanceUpdatedOn": 0,
            "instanceVersion": 0,
            "name": "string",
            "advancedPolicyScopeElement": [
              {
                "id": "string",
                "instanceId": 0,
                "displayName": "string",
                "instanceCreatedOn": 0,
                "instanceUpdatedOn": 0,
                "instanceVersion": 0,
                "groupId": [
                  "string"
                ],
                "ssid": [
                  {}
                ]
              }
            ]
          },
          "contractList": [
            {}
          ],
          "exclusiveContract": {
            "id": "string",
            "instanceId": 0,
            "displayName": "string",
            "instanceCreatedOn": 0,
            "instanceUpdatedOn": 0,
            "instanceVersion": 0,
            "clause": [
              {
                "id": "string",
                "instanceId": 0,
                "displayName": "string",
                "instanceCreatedOn": 0,
                "instanceUpdatedOn": 0,
                "instanceVersion": 0,
                "priority": 0,
                "type": "string",
                "relevanceLevel": "string",
                "deviceRemovalBehavior": "string",
                "hostTrackingEnabled": true
              }
            ]
          },
          "identitySource": {
            "id": "string",
            "instanceId": 0,
            "displayName": "string",
            "instanceCreatedOn": 0,
            "instanceUpdatedOn": 0,
            "instanceVersion": 0,
            "state": "string",
            "type": "string"
          },
          "producer": {
            "id": "string",
            "instanceId": 0,
            "displayName": "string",
            "instanceCreatedOn": 0,
            "instanceUpdatedOn": 0,
            "instanceVersion": 0,
            "scalableGroup": [
              {
                "idRef": "string"
              }
            ]
          },
          "consumer": {
            "id": "string",
            "instanceId": 0,
            "displayName": "string",
            "instanceCreatedOn": 0,
            "instanceUpdatedOn": 0,
            "instanceVersion": 0,
            "scalableGroup": [
              {
                "idRef": "string"
              }
            ]
          }
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: pxgrid_node_delete
short_description: Resource module for pxGrid Node Delete
description:
- Manage operation delete of the resource pxGrid Node Delete.
- This API deletes a pxGrid node by name.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  name:
    description: Name path parameter.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    px_grid_node.PxGridNode.delete_px_grid_node_by_name,

  - Paths used are
    delete /ers/config/pxgridnode/name/{name},

"""

EXAMPLES = r"""
- name: Delete by name
  cisco.ise.pxgrid_node_delete:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    name: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_virtual_media_policy
short_description: Virtual Media policy configuration for Cisco Intersight
description:
  - Virtual Media policy configuration for Cisco Intersight.
  - Used to configure Virtual Media image mappings on Cisco Intersight managed devices.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the NTP policy.
      - The name must be between 1 and 62 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  description:
    description:
      - The user-defined description of the NTP policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  enable:
    description:
      - Enable or disable virtual media.
    type: bool
    default: true
  encryption:
    description:
      - If enabled, allows encryption of all Virtual Media communications
    type: bool
    default: false
  low_power_usb:
    description:
      - If enabled, the virtual drives appear on the boot selection menu after mapping the image and rebooting the host.
    type: bool
    default: true
  cdd_virtual_media:
    description:
      - CDD Virtual Media image mapping options.
    type: dict
    suboptions:
      enable:
        description:
          - Enable or disable CDD image mapping.
        type: bool
        default: true
      mount_type:
        description:
          - Type (protocol) of network share used by the remote_hostname.
          - Ensure that the remote_hostname's communication port for the mount type that you choose is accessible from the managed endpoint.
          - For CIFS as your mount type, ensure port 445 (which is its communication port) on the remote_hostname is accessible.
          - For HTTP, ensure port 80 is accessible.
          - For HTTPS, ensure port 443 is accessible.
          - For NFS, ensure port 2049 is accessible.
        type: str
        choices: [nfs,cifs,http,https]
        required: true
      volume:
        description:
          - A user defined name of the image mounted for mapping.
        type: str
        required: true
      remote_hostname:
        description:
          - Hostname or IP address of the server hosting the virtual media image.
        type: str
        required: true
      remote_path:
        description:
          - Filepath (not including the filename) of the remote image.
          - Ex. mnt/SHARE/ISOS
        type: str
        required: true
      remote_file:
        description:
          - Filename of the remote image.
          - Ex. custom_image.iso
        type: str
        required: true
      username:
        description:
          - The username for the specified Mount Type, if required.
        type: str
      password:
        description:
          - The password for the selected username, if required.
        type: str
      mount_options:
        description:
          - Mount options for the Virtual Media mapping.
          - For NFS, supported options are ro, rw, nolock, noexec, soft, port=VALUE, timeo=VALUE, retry=VALUE
          - For CIFS, supported options are soft, nounix, noserverino, guest
        type: str
        required: false
      authentication_protocol:
        description:
          - Authentication Protocol for CIFS Mount Type
        type: str
        default: none
        required: false
  hdd_virtual_media:
    description:
      - HDD Virtual Media image mapping options.
    type: dict
    suboptions:
      enable:
        description:
          - Enable or disable HDD image mapping.
        type: bool
        default: true
      mount_type:
        description:
          - Type (protocol) of network share used by the remote_hostname.
          - Ensure that the remote_hostname's communication port for the mount type that you choose is accessible from the managed endpoint.
          - For CIFS as your mount type, ensure port 445 (which is its communication port) on the remote_hostname is accessible.
          - For HTTP, ensure port 80 is accessible.
          - For HTTPS, ensure port 443 is accessible.
          - For NFS, ensure port 2049 is accessible.
        type: str
        choices: [nfs,cifs,http,https]
        required: true
      volume:
        description:
          - A user defined name of the image mounted for mapping.
        type: str
        required: true
      remote_hostname:
        description:
          - Hostname or IP address of the server hosting the virtual media image.
        type: str
        required: true
      remote_path:
        description:
          - Filepath (not including the filename) of the remote image.
          - Ex. mnt/SHARE/ISOS
        type: str
        required: true
      remote_file:
        description:
          - Filename of the remote image.
          - Ex. custom_image.iso
        type: str
        required: true
      username:
        description:
          - The username for the specified Mount Type, if required.
        type: str
      password:
        description:
          - The password for the selected username, if required.
        type: str
      mount_options:
        description:
          - Mount options for the Virtual Media mapping.
          - For NFS, supported options are ro, rw, nolock, noexec, soft, port=VALUE, timeo=VALUE, retry=VALUE
          - For CIFS, supported options are soft, nounix, noserverino, guest
        type: str
        required: false
      authentication_protocol:
        description:
          - Authentication Protocol for CIFS Mount Type
        type: str
        default: none
        required: false
author:
  - David Soper (@dsoper2)
  - Sid Nath (@SidNath21)
'''

EXAMPLES = r'''
- name: Configure Virtual Media Policy
  cisco.intersight.intersight_virtual_media_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: DevNet
    name: lab-vmedia
    description: Virutal Media policy for lab use
    tags:
      - Key: Site
        Value: RCDN
    cdd_virtual_media:
      mount_type: nfs
      volume: nfs-cdd
      remote_hostname: 172.28.224.77
      remote_path: mnt/SHARE/ISOS/CENTOS
      remote_file: CentOS7.iso
    hdd_virtual_media:
      mount_type: nfs
      volume: nfs-hdd
      remote_hostname: 172.28.224.77
      remote_path: mnt/SHARE/ISOS/CENTOS
      remote_file: CentOS7.iso

- name: Delete Virtual Media Policy
  cisco.intersight.intersight_virtual_media_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: DevNet
    name: lab-vmedia
    state: absent
'''

RETURN = r'''
api_repsonse:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "lab-ntp",
        "ObjectType": "ntp.Policy",
        "Tags": [
            {
                "Key": "Site",
                "Value": "RCDN"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec, compare_values


def main():
    path = '/vmedia/Policies'
    virtual_media_mapping = dict(
        enable=dict(type='bool', default=True),
        mount_type=dict(type='str', choices=['nfs', 'cifs', 'http', 'https'], required=True),
        volume=dict(type='str', required=True),
        remote_hostname=dict(type='str', required=True),
        remote_path=dict(type='str', required=True),
        remote_file=dict(type='str', required=True),
        mount_options=dict(type='str'),
        username=dict(type='str'),
        password=dict(type='str', no_log=True),
        authentication_protocol=dict(type='str', default='none'),
    )
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        enable=dict(type='bool', default=True),
        encryption=dict(type='bool', default=False),
        low_power_usb=dict(type='bool', default=True),
        cdd_virtual_media=dict(type='dict', options=virtual_media_mapping),
        hdd_virtual_media=dict(type='dict', options=virtual_media_mapping),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''
    # Defined API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
        'Tags': intersight.module.params['tags'],
        'Description': intersight.module.params['description'],
        'Enabled': intersight.module.params['enable'],
        "Encryption": intersight.module.params['encryption'],
        "LowPowerUsb": intersight.module.params['low_power_usb'],
        'Mappings': [],
    }

    if intersight.module.params.get('cdd_virtual_media'):
        intersight.api_body['Mappings'].append(
            {
                "ClassId": "vmedia.Mapping",
                "ObjectType": "vmedia.Mapping",
                "AuthenticationProtocol": intersight.module.params['cdd_virtual_media']['authentication_protocol'],
                "DeviceType": "cdd",
                "HostName": intersight.module.params['cdd_virtual_media']['remote_hostname'],
                "Password": intersight.module.params['cdd_virtual_media']['password'],
                "IsPasswordSet": intersight.module.params['cdd_virtual_media']['password'] != '',
                "MountOptions": intersight.module.params['cdd_virtual_media']['mount_options'],
                "MountProtocol": intersight.module.params['cdd_virtual_media']['mount_type'],
                "RemoteFile": intersight.module.params['cdd_virtual_media']['remote_file'],
                "RemotePath": intersight.module.params['cdd_virtual_media']['remote_path'],
                "Username": intersight.module.params['cdd_virtual_media']['username'],
                "VolumeName": intersight.module.params['cdd_virtual_media']['volume'],
            }
        )
    if intersight.module.params.get('hdd_virtual_media'):
        intersight.api_body['Mappings'].append(
            {
                "ClassId": "vmedia.Mapping",
                "ObjectType": "vmedia.Mapping",
                "AuthenticationProtocol": intersight.module.params['hdd_virtual_media']['authentication_protocol'],
                "DeviceType": "hdd",
                "HostName": intersight.module.params['hdd_virtual_media']['remote_hostname'],
                "Password": intersight.module.params['hdd_virtual_media']['password'],
                "IsPasswordSet": intersight.module.params['hdd_virtual_media']['password'] != '',
                "MountOptions": intersight.module.params['hdd_virtual_media']['mount_options'],
                "MountProtocol": intersight.module.params['hdd_virtual_media']['mount_type'],
                "RemoteFile": intersight.module.params['hdd_virtual_media']['remote_file'],
                "RemotePath": intersight.module.params['hdd_virtual_media']['remote_path'],
                "Username": intersight.module.params['hdd_virtual_media']['username'],
                "VolumeName": intersight.module.params['hdd_virtual_media']['volume'],
            }
        )

    organization_moid = None
    # GET Organization Moid
    intersight.get_resource(
        resource_path='/organization/Organizations',
        query_params={
            '$filter': "Name eq '" + intersight.module.params['organization'] + "'",
            '$select': 'Moid',
        },
    )
    if intersight.result['api_response'].get('Moid'):
        # resource exists and moid was returned
        organization_moid = intersight.result['api_response']['Moid']

    intersight.result['api_response'] = {}
    # get the current state of the resource
    filter_str = "Name eq '" + intersight.module.params['name'] + "'"
    filter_str += "and Organization.Moid eq '" + organization_moid + "'"
    intersight.get_resource(
        resource_path=path,
        query_params={
            '$filter': filter_str,
            '$expand': 'Organization',
        },
    )

    moid = None
    resource_values_match = False
    if intersight.result['api_response'].get('Moid'):
        # resource exists and moid was returned
        moid = intersight.result['api_response']['Moid']
        if module.params['state'] == 'present':
            resource_values_match = compare_values(intersight.api_body, intersight.result['api_response'])
        else:  # state == 'absent'
            intersight.delete_resource(
                moid=moid,
                resource_path=path,
            )
            moid = None

    if module.params['state'] == 'present' and not resource_values_match:
        # remove read-only Organization key
        intersight.api_body.pop('Organization')
        if not moid:
            # Organization must be set, but can't be changed after initial POST
            intersight.api_body['Organization'] = {
                'Moid': organization_moid,
            }
        intersight.configure_resource(
            moid=moid,
            resource_path=path,
            body=intersight.api_body,
            query_params={
                '$filter': filter_str,
            },
        )

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: interface_operation_create
short_description: Resource module for Interface Operation Create
description:
- This module represents an alias of the module interface_operation_create_v1
version_added: '6.0.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deploymentMode:
    description: DeploymentMode query parameter. Preview/Deploy 'Preview' means the
      configuration is not pushed to the device. 'Deploy' makes the configuration pushed
      to the device.
    type: str
  interfaceUuid:
    description: InterfaceUuid path parameter. Interface Id.
    type: str
  operation:
    description: Operation needs to be specified as 'ClearMacAddress'.
    type: str
  payload:
    description: Payload is not applicable.
    type: dict
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices ClearMacAddressTableV1
  description: Complete reference of the ClearMacAddressTableV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!clear-mac-address-table
notes:
  - SDK Method used are
    devices.Devices.clear_mac_address_table_v1,

  - Paths used are
    post /dna/intent/api/v1/interface/{interfaceUuid}/operation,
  - It should be noted that this module is an alias of interface_operation_create_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.interface_operation_create:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deploymentMode: string
    interfaceUuid: string
    operation: string
    payload: {}

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: health_score_definitions
short_description: Resource module for Health Score Definitions
description:
- This module represents an alias of the module health_score_definitions_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Health score definition id.
    type: str
  includeForOverallHealth:
    description: Include For Overall Health.
    type: bool
  synchronizeToIssueThreshold:
    description: Synchronize To Issue Threshold.
    type: bool
  thresholdValue:
    description: Thresehold Value.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices UpdateHealthScoreDefinitionForTheGivenIdV1
  description: Complete reference of the UpdateHealthScoreDefinitionForTheGivenIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-health-score-definition-for-the-given-id
notes:
  - SDK Method used are
    devices.Devices.update_health_score_definition_for_the_given_id_v1,

  - Paths used are
    put /dna/intent/api/v1/healthScoreDefinitions/{id},
  - It should be noted that this module is an alias of health_score_definitions_v1

"""

EXAMPLES = r"""
- name: Update by id
  cisco.dnac.health_score_definitions:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    id: string
    includeForOverallHealth: true
    synchronizeToIssueThreshold: true
    thresholdValue: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "displayName": "string",
      "deviceFamily": "string",
      "description": "string",
      "includeForOverallHealth": true,
      "definitionStatus": "string",
      "thresholdValue": 0,
      "synchronizeToIssueThreshold": true,
      "lastModified": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The iosxr bgp_neighbor_address_family fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.bgp_neighbor_address_family.bgp_neighbor_address_family import (
    Bgp_neighbor_address_familyArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.bgp_neighbor_address_family import (
    Bgp_neighbor_address_familyTemplate,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.utils.utils import (
    flatten_config,
)


class Bgp_neighbor_address_familyFacts(object):
    """The iosxr bgp_neighbor_address_family facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Bgp_neighbor_address_familyArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_config(self, connection):
        return connection.get("show running-config router bgp")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Bgp_address_family network resource
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []
        if not data:
            data = self.get_config(connection)
        nbr_data = flatten_config(data, "neighbor")
        data = flatten_config(nbr_data, "vrf")
        # parse native config using the Bgp_global template
        bgp_global_parser = Bgp_neighbor_address_familyTemplate(
            lines=data.splitlines(),
        )
        objs = bgp_global_parser.parse()

        if objs:
            top_lvl_nbrs = objs.get("vrfs", {}).pop("vrf_", {})
            objs["neighbors"] = self._post_parse(top_lvl_nbrs).get(
                "neighbors",
                [],
            )

            if "vrfs" in objs:
                for vrf in objs["vrfs"].values():
                    vrf["neighbors"] = self._post_parse(vrf)["neighbors"]
                objs["vrfs"] = list(objs["vrfs"].values())

        ansible_facts["ansible_network_resources"].pop(
            "bgp_neighbor_address_family",
            None,
        )

        params = utils.remove_empties(
            utils.validate_config(self.argument_spec, {"config": objs}),
        )

        facts["bgp_neighbor_address_family"] = params.get("config", {})
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts

    def _post_parse(self, data):
        """Converts the intermediate data structure
            to valid format as per argspec.
        :param obj: dict
        """
        if "neighbors" in data:
            data["neighbors"] = sorted(
                list(data["neighbors"].values()),
                key=lambda k, s="neighbor_address": k[s],
            )
            for nbr in data["neighbors"]:
                nbr["address_family"] = list(nbr["address_family"].values())
        return data



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: site_health_v1_info
short_description: Information module for Site Health V1
description:
- Get all Site Health V1.
- Returns Overall Health information for all sites.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  siteType:
    description:
    - SiteType query parameter. Site type AREA or BUILDING (case insensitive).
    type: str
  offset:
    description:
    - Offset query parameter. Offset of the first returned data set entry (Multiple of 'limit' + 1).
    type: float
  limit:
    description:
    - Limit query parameter. Max number of data entries in the returned data set 1,50. Default is 25.
    type: float
  timestamp:
    description:
    - Timestamp query parameter. Epoch time(in milliseconds) when the Site Hierarchy data is required.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Sites GetSiteHealthV1
  description: Complete reference of the GetSiteHealthV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-site-health
notes:
  - SDK Method used are
    sites.Sites.get_site_health_v1,

  - Paths used are
    get /dna/intent/api/v1/site-health,

"""

EXAMPLES = r"""
- name: Get all Site Health V1
  cisco.dnac.site_health_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    siteType: string
    offset: 0
    limit: 0
    timestamp: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "siteName": "string",
        "siteId": "string",
        "parentSiteId": "string",
        "parentSiteName": "string",
        "siteType": "string",
        "latitude": 0,
        "longitude": 0,
        "healthyNetworkDevicePercentage": 0,
        "healthyClientsPercentage": 0,
        "clientHealthWired": 0,
        "clientHealthWireless": 0,
        "numberOfClients": 0,
        "numberOfNetworkDevice": 0,
        "networkHealthAverage": 0,
        "networkHealthAccess": 0,
        "networkHealthCore": 0,
        "networkHealthDistribution": 0,
        "networkHealthRouter": 0,
        "networkHealthWireless": 0,
        "networkHealthAP": 0,
        "networkHealthWLC": 0,
        "networkHealthSwitch": 0,
        "networkHealthOthers": 0,
        "numberOfWiredClients": 0,
        "numberOfWirelessClients": 0,
        "totalNumberOfConnectedWiredClients": 0,
        "totalNumberOfActiveWirelessClients": 0,
        "wiredGoodClients": 0,
        "wirelessGoodClients": 0,
        "overallGoodDevices": 0,
        "accessGoodCount": 0,
        "accessTotalCount": 0,
        "coreGoodCount": 0,
        "coreTotalCount": 0,
        "distributionGoodCount": 0,
        "distributionTotalCount": 0,
        "routerGoodCount": 0,
        "routerTotalCount": 0,
        "wirelessDeviceGoodCount": 0,
        "wirelessDeviceTotalCount": 0,
        "apDeviceGoodCount": 0,
        "apDeviceTotalCount": 0,
        "wlcDeviceGoodCount": 0,
        "wlcDeviceTotalCount": 0,
        "switchDeviceGoodCount": 0,
        "switchDeviceTotalCount": 0,
        "applicationHealth": 0,
        "applicationHealthInfo": [
          {
            "trafficClass": "string",
            "bytesCount": 0,
            "healthScore": 0
          }
        ],
        "applicationGoodCount": 0,
        "applicationTotalCount": 0,
        "applicationBytesTotalCount": 0,
        "dnacInfo": {
          "uuid": "string",
          "ip": "string",
          "status": "string"
        },
        "usage": 0,
        "applicationHealthStats": {
          "appTotalCount": 0,
          "businessRelevantAppCount": {
            "poor": 0,
            "fair": 0,
            "good": 0
          },
          "businessIrrelevantAppCount": {
            "poor": 0,
            "fair": 0,
            "good": 0
          },
          "defaultHealthAppCount": {
            "poor": 0,
            "fair": 0,
            "good": 0
          }
        }
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: clients_query_count_v1
short_description: Resource module for Clients Query Count V1
description:
- Manage operation create of the resource Clients Query Count V1.
- >
   Retrieves the number of clients by applying complex filters. For detailed information about the usage of the API,
   please refer to the Open API specification document - https //github.com/cisco-en-programmability/catalyst-center-
   api-specs/blob/main/Assurance/CE_Cat_Center_Org-clients1-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  endTime:
    description: End Time.
    type: int
  filters:
    description: Clients Query Count's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: int
    type: list
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description: Start Time.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Clients RetrievesTheNumberOfClientsByApplyingComplexFiltersV1
  description: Complete reference of the RetrievesTheNumberOfClientsByApplyingComplexFiltersV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieves-the-number-of-clients-by-applying-complex-filters
notes:
  - SDK Method used are
    clients.Clients.retrieves_the_number_of_clients_by_applying_complex_filters_v1,

  - Paths used are
    post /dna/data/api/v1/clients/query/count,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.clients_query_count_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    endTime: 0
    filters:
    - key: string
      operator: string
      value: 0
    headers: '{{my_headers | from_json}}'
    startTime: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: duo_mfa_info
short_description: Information module for Duo Mfa Info
description:
- Get all Duo Mfa Info.
- Get Duo Mfa Info by name.
- Duo-MFA - Get the Duo-MFA configuration specified in the connectionName.
- Duo-MFA - List of Duo-MFA configurations.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  connectionName:
    description:
    - ConnectionName path parameter. This name is used to update, delete or retrieve the specific Duo-MFA configuration.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    duo_mfa.DuoMfa.get_mfa,
    duo_mfa.DuoMfa.get_mfa_byconnection_name,

  - Paths used are
    get /api/v1/duo-mfa/mfa,
    get /api/v1/duo-mfa/mfa/{connectionName},

"""

EXAMPLES = r"""
- name: Get all Duo Mfa Info
  cisco.ise.duo_mfa_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
  register: result

- name: Get Duo Mfa Info by name
  cisco.ise.duo_mfa_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    connectionName: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "mfa": {
        "accountConfigurations": {
          "adminApi": {
            "ikey": "string",
            "sKey": "string"
          },
          "apiHostName": "string",
          "authenticationApi": {
            "ikey": "string",
            "sKey": "string"
          }
        },
        "connectionName": "string",
        "description": "string",
        "identitySync": "string",
        "type": "string"
      }
    }
"""
